# $Id: CGIHandler.py 2028 2006-01-16 23:39:04Z zzzeek $
# CGIHandler.py - handles cgi requests for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

import myghty.interp
import myghty.request
import myghty.resolver
import myghty.session as session
import myghty.exception as exception
import myghty.http.HTTPHandler as HTTPHandler
from myghty.util import *
import os, sys, cgi


def handle(*args, **params):
    return HTTPHandler.handle_http(CGIHandler, *args, **params)

def get_handler(interpreter_name = None, **params):
        return HTTPHandler.get_handler(CGIHandler, interpreter_name = interpreter_name, **params)

class CGIHandler(HTTPHandler.HTTPHandler):

    def __init__(self, **params):
        HTTPHandler.HTTPHandler.__init__(self, None, sys.stderr, **params)

    def do_get_init_params(self, httpreq, **params):
        return params
    
    def do_get_resolver(self, **params):
        return CGIResolver(**params)
    
    def do_make_request_impl(self, httpreq, **params):
        return CGIRequestImpl(httpreq, **params)
        
    def do_get_component(self, httpreq, **params):
        return httpreq.path_info
        
    def do_handle_result(self, httpreq, status_code, reason):
        httpreq.send_http_header()
        if status_code != HTTPHandler.HTTP_OK:
            sys.stdout.write("""
                <html>
                <body>
                <b>Response code %d (%s)</b><br/>
                Page: "%s"
                </body>
                </html>
                """  % (status_code, reason or "no message", httpreq.path_info))


    def handle(self, httpreq = None, interp = None, request_impl = None, component = None, **params):
        if httpreq is None:
            httpreq = CGIHTTPRequest()
            
        HTTPHandler.HTTPHandler.handle(self, httpreq, interp, request_impl, component, **params)

class CGIHTTPRequest(HTTPHandler.HTTPRequest):
    """simulates a mod_python request for use in non-mod-python applications.
    only a minimal featureset is here currently.
    """
    def __init__(self):
        HTTPHandler.HTTPRequest.__init__(self)

        self.fieldstorage = cgi.FieldStorage(keep_blank_values = True)

        env = os.environ
        for key in env.keys():
            if key[0:4] == 'HTTP':
                self.headers_in.add(key[5:], env[key])
                        
        self.content_type = 'text/html'
        self.method = env['REQUEST_METHOD']
        self.filename = env['PATH_TRANSLATED']
        self.args = env.get('QUERY_STRING', None)
        self.path_info = env.get('PATH_INFO', '/')
        self.out_buffer = sys.stdout
        
    def do_send_headers(self):
        headers = self.get_response_headers()
        headers.get_output(self.out_buffer)
        self.out_buffer.write("\n")



class CGIResolver(HTTPHandler.HTTPResolver):pass

class CGIWriter(HTTPHandler.HTTPWriter):pass
        

class CGIRequestImpl(HTTPHandler.HTTPRequestImpl):
    def __init__(self, httpreq, **params):
        HTTPHandler.HTTPRequestImpl.__init__(self, httpreq, sys.stderr, sys.stderr, **params)

    def do_get_out_buffer(self, httpreq, out_buffer = None, **params):
        if out_buffer is not None:
            return CGIWriter(httpreq, out_buffer)
        else:
            return CGIWriter(httpreq, httpreq.out_buffer)

    def do_make_request_args(self, httpreq, **params):
        return self.request_args_from_fieldstorage(httpreq.fieldstorage)





#!/usr/bin/env python

import sys

try:
    import optparse
except:
    sys.stderr.write("gen.py requires the optparse module, available in python 2.3 or higher\n")
    sys.exit(-1)

import myghty.interp
import StringIO, re, os.path

parser = optparse.OptionParser(usage = "usage: %prog [options] files...")
parser.add_option("--croot", action="store", dest="component_root", default=".", help="set component root (default: ./)")
parser.add_option("--dest", action="store", dest="destination", default=".", help="set destination directory (default: ./)")
parser.add_option("--stdout", action="store_true", dest="stdout",  help="send output to stdout")
parser.add_option("--datadir", action="store", dest="datadir", help="set data directory (default: dont use data directory)")
parser.add_option("--ext", action="store", dest="extension", default=".html", help="file extension for output files (default: .html)")
parser.add_option("--source", action="store_true", dest="source", help="generate the source component to stdout")

(options, args) = parser.parse_args()


params = {}
if options.datadir:
    params['data_dir'] = options.datadir

interp = myghty.interp.Interpreter(component_root = options.component_root,
                **params
                )

if not len(args):
    parser.print_help()

for arg in args:
    
    if options.source:
        source = interp.resolver.get_component_source(arg)
        source.get_object_code(interp.compiler(), sys.stdout)
        continue

    elif options.stdout:
        outbuf = sys.__stdout__
    else:
        (dir, name) = os.path.split(arg)
        if options.destination:
            dir = options.destination
            
        outfile = re.sub(r"\..+$", "%s" % options.extension, name)
        outfile = os.path.join(dir, outfile)
        print "%s -> %s" % (arg, outfile)
        outbuf = open(outfile, "w")
        

    interp.execute(arg, out_buffer = outbuf)
    
    if not options.stdout:
        outbuf.close()


    







"""
tools/run_docs.py documentation runner.  

*** ADVANCED EXAMPLE !!! PUSHES CONFIG TO THE LIMIT ! ***

for a simpler standalone example, please consult /examples/shoppingcart/run_cart.py .

"""

import os, sys, re

# adjust path to use local directory
[sys.path.insert(0, path) for path in ['./lib', './doc/lib', './examples/common', './examples/shoppingcart/lib', './examples/formvisitor/lib']]

# myghty imports
import myghty.http.HTTPServerHandler as HTTPServerHandler
from myghty.resolver import *

# determine port number
try:
    port = int(sys.argv[1])
except:
    port = 8000

# create local cache directory to store generated files + sessions
if not os.access('./cache', os.F_OK):
    os.mkdir('./cache')


# now set up standalone server    
httpd = HTTPServerHandler.HTTPServer(
    port = port,

    # send all request content types through the full resolution chain
    text_only = False,
    
    # handlers.  a list of regular expressions matched to directories to serve
    # or Interpreter configurations.
    handlers = [
        {r'.*/$|.*\.myt$|/source/.*|/examples/shoppingcart/store/.*' : HTTPServerHandler.HSHandler(
            data_dir = './cache',

            #use_static_source = True,
            
            #debug_elements = ['resolution'],

            #disable_unicode=True, 
            resolver_strategy = [
                # caching.  if static_source is turned on, everything below this rule
                # is cached based on the URI as well as contextual modifiers.
                URICache(),

                # create a group that handles all shopping cart URIs
                ConditionalGroup(regexp='/examples/shoppingcart/store/.*', rules = [

                    # create a "request-only" subgroup for resolving the lead module
                    # components.  this isolates request-level resolutions from
                    # subrequest and component resolutions, greatly reducing the
                    # overall number of lookups required per-component.
                    ConditionalGroup(context='request', rules = [
                        #ResolveModule(
                        #         {r'.*/catalog/.*' : 'shoppingcontroller:index.catalog'},
                        #         {r'.*/item/.*' : 'shoppingcontroller:index.item'},
                        #         {r'.*/cart/.*' : 'shoppingcontroller:index.cart'},
                        #         {r'.*/source/.*' : 'shoppingcontroller:index.source'},
                        #     ),
                        ResolvePathModule(
                           'shoppingcontroller',
                           adjust = lambda u: re.sub(r'/examples/shoppingcart/store/', '/', u)
                        ),

                    ]),
    
                    ResolveDhandler(),
                    ResolveUpwards(),
                    ResolveFile(
                        {'store_comp':'./examples/shoppingcart/components'},
                        {'store_templ':'./examples/shoppingcart/templates'},
                        {'store_htdocs':'./examples/shoppingcart/htdocs'},
                       adjust = lambda u: re.sub(r'/examples/shoppingcart/store/', '/', u)
                    ),
                    
                    NotFound()
                ]),
                
                # resolution rules for the /source/ browser
                Conditional(
                    ResolveModule({r'.*' : 'modulecomponents:ViewSource'}),
                    regexp='/source/.*'
                ),
                
                # resolution rules for the documentation viewer
                ConditionalGroup(regexp='^/doc/$|^/doc/.*\.myt$', rules = [
                    PathTranslate(
                        (r'^/doc/$', '/doc/index.myt'),
                    ),
                    
                    # upwards search, i.e. for /autohandler.  the remaining rules
                    # in this chain will be used to match against successive 
                    # /autohandler uris.
                    ResolveUpwards(),
                    
                    AdjustedResolveFile(
                            [(r'^/doc/', r'/')],
                            {'common':'./examples/common'},
                            {'doc_comp':'./doc/components'},
                            {'doc_content':'./doc/content'},
                    ),
                    
                    NotFound()
                ]),

                # default resolution rules, after the above conditionals have not been
                # met
                
                PathTranslate(
                    (r'^/examples', ''),
                    (r'/$', '/index.myt'),
                ),
                
                # if a dhandler is being searched, the remainder of the rules will 
                # be queried with successive /dhandler uris
                ResolveDhandler(),
                ResolveUpwards(),

                ResolveFile(
                    {'common':'./examples/common'},
                    {'example_htdocs':'./examples'},
                ),
            ],
            
            attributes = {
                'store_uri' : '/examples/shoppingcart/store/',
                'store_document_uri' : '/examples/shoppingcart/docs/',
                'store_path' : '/examples/shoppingcart/store/',

                'source_uri' : '/source/',
                'source_root' : './',
                'docs_static_cache': True,
            },
            
        )
        
        },

        # docroots
        {r'/examples/shoppingcart/docs/(.*)' : './examples/shoppingcart/htdocs'},
        {r'/examples/shoppingcart/store/source/(.*)' : './examples/shoppingcart/'},
        {r'/doc/(.*)' : './doc/html'},
        {r'/examples/(.*)' : './examples'},
        {r'/source/(.*)' : '.'},
        {r'/(.*)':'./examples'},


    ]
)
    
print "HTTPServer listening on port %d" % httpd.port


httpd.serve_forever()




import zblog
import zblog.controller
import zblog.domain.actions as actions
import re
import myghty.exception as exception

class FrontController(zblog.controller.Controller):
    """initial controller accessed for all pages.
    
    Is used for access control, as well as initial application configuration 
    is loaded via Interpreter attributes.  If configuration file doesnt 
    exist, bounces to "bootstrap" page to create config file.  """

    def check_bootstrap(self, m):
        """checks if we are in "bootstrap mode", which means theres no config   
        or database setup yet."""
        bootstrap = m.resolution.match.group(1).startswith('/bootstrap/')
        if zblog.need_config and not bootstrap:
            m.send_redirect('/bootstrap/', hard=True)
        elif not zblog.need_config and bootstrap:
            m.abort(403)
            
    def __call__(self, m, ARGS):
        """main handling method. does coarse-grained access control check and 
        forwards on to the requested controller."""
        self.check_bootstrap(m)

        if not zblog.need_config:
            # start mapper session.  clears out previous identitymaps and units of
            # work, within the current thread
            zblog.database.mappers.start_session()

        uri = m.resolution.match.group(1)
        try:
            controller = m.fetch_component(uri, resolver_context="frontcontroller", enable_dhandler=True)
        except exception.ComponentNotFound, cfound:
            raise cfound.create_toplevel()
        
        public = getattr(controller.component_source.callable_, 'public', False)
        if not public:
            m.abort(404)
            return
        
        user = self.get_user(m)
        login = getattr(controller.component_source.callable_, 'login', False)
        if login and user is None:
            login = m.fetch_component('/login/', resolver_context="frontcontroller", enable_dhandler=True)
            m.subexec(login)
            return
            
        action = getattr(controller.component_source.callable_, 'action', None)
        if action is not None:
            if not action.access(user, **ARGS):
                m.abort(403)
                return

        m.comp(controller, **m.request_args)
        

        
index = FrontController()




# $Id: __init__.py 2013 2005-12-31 03:19:39Z zzzeek $
# __init__.py 
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#


import posixpath as unixpath

# insert the "http" path into the search path for this module
__path__.insert(0, unixpath.join(__path__[0], 'http'))






# $Id: resolver.py 2024 2006-01-12 04:16:15Z zzzeek $
# resolver.py - file path resolution functions for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz. 
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#


import types, re, os, string, stat, sys, imp
import posixpath as unixpath
import myghty.util as util
from myghty import exception
import myghty.csource as csource
import myghty.importer as importer
import time, copy

__all__ = ['Resolver', 'ResolverRule', 'ResolveFile', 'AdjustedResolveFile', 'ResolveModule', 'Group', 'Conditional', 'ConditionalGroup', 
        'NotFound', 'PathTranslate', 'ResolveDhandler', 'ResolveUpwards', 'URICache', 'ResolvePathModule']

class Resolution(object):
    """represents a resolved uri, including the component source object, 
    a list of resolution detail messages, and an 
    optional regexp match object."""
    def __init__(self, csource, resolution_detail, match = None, can_cache = True, **params):
        self.csource = csource
        self.detail = resolution_detail
        self.match = match
        self.can_cache = can_cache
        for key, value in params.iteritems():
            setattr(self, key, value)

class ResolverRule(object):
    """base class for a single rule that attempts to resolve a uri into a Resolution 
    object.  acts inside a chain of ResolverRules where when a uri cannot be resolved,
    the next rule in the chain is called in a continuation pattern."""
    
    def init_resolver(self, resolver, remaining_rules, **params):
        if hasattr(self, 'resolver'): raise exception.ConfigurationError("resolver rule %s already initialized with a Resolver" % repr(self))
        self.resolver = resolver
        self.do_init_resolver(resolver, remaining_rules, **params)
        
    def do_init_resolver(self, resolver, remaining_rules, **params):
        """ called when the resolver first receives the new rule.
        objects usually will want to do most of their initialization
        at this stage. """
        pass
 
    def do(self, uri, remaining, resolution_detail, **params):
        """ performs resolution or translation on the given path.  
        "remaining" is an Iterator referencing an element in the full 
        list of ResolverRule objects.  the method either returns a 
        Resolution object, or passes control to the next
        ResolverRule in the iterator."""
        raise NotImplementedError()

    def modifies_uri(self):
        """ returns True if the rule makes a modification to the URI.  
        if so, the URICache rule wont allow this rule to be cached as a single
        rule, since it becomes ineffective.  As part of a cached chain is still
        OK."""
        return False

class ResolveFile(ResolverRule):
    """performs file based resolution. looks up files within one or
    more component roots."""
    
    name = 'resolvefile'
    
    def __init__(self, *component_roots, **params):
        self._adjust = params.get('adjust', None) or params.get('component_root_adjust', None)
        
        if len(component_roots):
            self.component_root = component_roots
        else:
            self.component_root = None
    
    def do_init_resolver(self, resolver, remaining_rules, component_root = None, **params):
        if self.component_root is None:
            self.component_root = component_root

        if self.component_root is None: return

        if self._adjust is None:
            self._adjust = params.get('component_root_adjust', None)
        
        if not isinstance(self.component_root, types.ListType) and not isinstance(self.component_root, types.TupleType):
            self.component_root = [{'main':self.component_root}]
        
        self.component_root = util.OrderedDict(self.component_root)

    def do(self, uri, remaining, resolution_detail, **params):
        if self.component_root is None: return remaining.next().do(uri, remaining, resolution_detail, **params)
        
        if resolution_detail is not None: resolution_detail.append("resolvefile:")
        
        if self._adjust is not None:
            path = self._adjust(uri)
            if resolution_detail is not None: resolution_detail.append("adjust: %s -> %s" % (uri, path))
        else:
            path = uri
            
        # internal path, trim off leading '/'
        if path and path[0] == '/':
            path = path[1:]
        
        for key, root in self.component_root.iteritems():
            srcfile = unixpath.join(root, path)
            
            if resolution_detail is not None: resolution_detail.append(srcfile)
            
            if os.access(srcfile, os.F_OK):
                break
        else:
            return remaining.next().do(uri, remaining, resolution_detail, **params)
            
        st = os.stat(srcfile)
        (mode, modtime) = (st[stat.ST_MODE], st[stat.ST_MTIME])
        
        if stat.S_ISDIR(mode):
            if resolution_detail is not None: 
                resolution_detail.append("isdirectory: " + uri)
            return remaining.next().do(uri, remaining, resolution_detail, **params)
        
        # the resolved uri, insure theres a leading '/'
        if not uri or uri[0] != '/':
            uri = '/' + uri
            
        return Resolution(
                csource.FileComponentSource(
                    file_path = srcfile,
                    last_modified = modtime,
                    path = uri,
                    path_id = key,
                    id = "%s|%s" % (key, path), 
                ), 
                resolution_detail
            )

        

class ResolvePathModule(ResolverRule):
    """resolves a callable object or class instance inside a module, based on a traversal
    of path tokens corresponding to the module's file location, and then of the object paths
    defined within the module."""
    name = 'pathmodule'
    
    def __init__(self, *module_roots, **params):
        self._adjust = params.get('adjust', None) or params.get('module_root_adjust', None)
        self._require_publish = params.get('require_publish', None)
        self.stringtokens = params.get('path_stringtokens', None)
        self.moduletokens = params.get('path_moduletokens', None)
        
        if len(module_roots):
            self.module_root = list(module_roots)
        else:
            self.module_root = None

    def do_init_resolver(self, resolver, remaining_rules, module_root = None, **params):
        self.use_static_source = resolver.use_static_source

        if self.module_root is None:
            self.module_root = module_root
        if self.module_root is None: return

        if self._adjust is None:
            self._adjust = params.get('module_root_adjust', None)
        if self._require_publish is None:
            self._require_publish = params.get('require_publish', False)
        if self.stringtokens is None:
            self.stringtokens = params.get('path_stringtokens', [])
        if self.moduletokens is None:
            self.moduletokens = params.get('path_moduletokens', ['index'])
        
        for i in range(0, len(self.module_root)):
            value = self.module_root[i]
            if not isinstance(value, types.ModuleType):
                try:
                    st = os.stat(value)
                    if stat.S_ISREG(st[stat.ST_MODE]):
                        self.module_root[i] = importer.filemodule(value)
                except OSError:
                    try:
                        self.module_root[i] = importer.module(value)
                    except Exception, e:
                        raise exception.ConfigurationError("Path %s is not a file, directory, or importable module (%s: %s)" % (value, e.__class__.__name__, e.args[0]))
                
    def do(self, uri, remaining, resolution_detail, dhandler_path = None, **params):
        if self.module_root is None: return remaining.next().do(uri, remaining, resolution_detail, **params)
        
        if resolution_detail is not None: resolution_detail.append("resolvemodulepath:")
        
        if self._adjust is not None:
            path = self._adjust(uri)
            if resolution_detail is not None: resolution_detail.append("adjust: %s -> %s" % (uri, path))
        else:
            path = uri
        
        # internal path, trim off leading '/'
        if path and path[0] == '/':
            path = path[1:]

        if dhandler_path is not None:
            path = path + '/' + dhandler_path

        if (path == '/' or not path) and len(self.stringtokens):
            path= '/' + self.stringtokens[0]
            
        for root in self.module_root:
            if resolution_detail is not None:
                resolution_detail.append(repr(root) + "/" + path)
                
            iterator = importer.ObjectPathIterator(root, reload = not self.use_static_source)
            unit = root
            for token in path.split('/'):
                if not token:
                    continue
                if len(token) > 0 and token[0] == '_':
                    token = re.sub(r'^_+', '', token)
                try:
                    while True:
                        (unit, matched) = iterator.get_unit([token], stringtokens = self.stringtokens, moduletokens = self.moduletokens)
                        if matched == token:
                            break
                except StopIteration:
                    pass
            
            if unit is not None:
                while unit is not None and (isinstance(unit, types.ModuleType) or isinstance(unit, str)):
                    try:
                        (unit,matched) = iterator.get_unit([], stringtokens=self.stringtokens, moduletokens=self.moduletokens)
                        if unit is None:
                            break
                    except StopIteration:
                        break
                    
                if isinstance(unit, types.FunctionType) or isinstance(unit, types.MethodType) or callable(unit):
                    if self._require_publish and not getattr(unit, 'publish', False):
                        return remaining.next().do(uri, remaining, resolution_detail, **params)
                    cs = csource.ModuleComponentSource(module = iterator.module, objpath = iterator.objpath, last_modified = iterator.last_modified, arg = unit)
                    break
                else:
                    continue
        else:
            return remaining.next().do(uri, remaining, resolution_detail, **params)

        return Resolution(cs, resolution_detail)


def adjust_path(adjust, path):
    for a in adjust:
        path = re.sub(a[0], a[1], path)
    return path

class AdjustedResolveFile(ResolveFile):
    """a ResolveFile rule that adds a pre-path-concatenation adjustment step, so the uri can
    be translated before determining the file path.  unlike path_translate, this translated
    uri is only used to determine a filesystem path, and is not propigated anywhere else."""
    def __init__(self, adjust, *component_roots):
        ResolveFile.__init__(self, adjust = lambda p: adjust_path(adjust, p), *component_roots)

class AdjustedResolvePathModule(ResolvePathModule):
    """a ResolvePathModule rule that adds a pre-path-concatenation adjustment step, so the uri can
    be translated before determining the file path.  unlike path_translate, this translated
    uri is only used to determine a filesystem path, and is not propigated anywhere else."""
    def __init__(self, adjust, *module_roots):
        ResolvePathModule.__init__(self, adjust = lambda p: adjust_path(adjust, p), *module_roots)
        
class ResolveModule(ResolverRule):
    """resolves a Module Component based on information in a given list of dictionaries,
    containing regular expressions for keys which are matched against the URI. The value 
    for each key references either a class, a callable object or function, or a string in 
    the form "<module>:<path.to.callable>"."""
    
    name = 'resolvemodule'
    
    def __init__(self, *module_components):
        if len(module_components):
            self.module_components = module_components
        else:
            self.module_components = None
        self.csource_cache = {}
        
    def do_init_resolver(self, resolver, remaining_rules, module_components = None, **params):
        self.use_static_source = resolver.use_static_source
        if self.module_components is None:
            self.module_components = module_components

        if self.module_components is None: return
        
        self.module_components = util.OrderedDict(self.module_components)
        for key, value in self.module_components.iteritems():
            if not isinstance(value, types.DictType):
                value = {'component':value}
                self.module_components[key] = value

    def do(self, uri, remaining, resolution_detail, context = None, **params):
        if self.module_components is None: return remaining.next().do(uri, remaining, resolution_detail, **params)

        if resolution_detail is not None: resolution_detail.append("resolvemodule: " + uri)
        
        for reg in self.module_components.keys():
            match = re.match(reg, uri)
            if match:
                info = self.module_components[reg]
                compsource = csource.ModuleComponentSource(arg = info['component'], use_static_source = self.use_static_source, arg_cache=self.csource_cache)
                return Resolution(
                    compsource,
                    resolution_detail,
                    match,
                    args = info
                    )
        else:
            return remaining.next().do(uri, remaining, resolution_detail, **params)
                

class Group(ResolverRule):
    """creates a subgroup of resolver strategies."""
    def __init__(self, rules = []):
        self.rules = rules

    def do_init_resolver(self, resolver, remaining_rules, **params):
        for i in range(0, len(self.rules)):
            self.rules[i].init_resolver(resolver, self.rules[i + 1:], **params)
                                 
    def do(self, uri, remaining, resolution_detail, **params):
        def iterator():
            for rule in self.rules:
                yield rule
            for rule in remaining:
                yield rule
        i = iterator()
        return i.next().do(uri, i, resolution_detail, **params)

    
class Conditional(ResolverRule):
    """conditionally executes a rule, only executes if the uri matches a certain regexp,
    or a passed-in context string matches one of the contexts set up for this rule."""
    
    def __init__(self, rule, regexp = None, context = None):
        self.rule = rule
        
        if context is not None:
            self.contexts = context.split(',')
        else:
            self.contexts = []
        self.regexp = regexp

    def do_init_resolver(self, resolver, remaining_rules, **params):
        self.rule.init_resolver(resolver, remaining_rules, **params)

    def _match_context(self, context):
        for c in self.contexts:
            if c == context: return True
        return False

    
    def test_condition(self, uri, resolver_context, **params):
        return (
            (resolver_context is not None and self._match_context(resolver_context))
            or
            (self.regexp is not None and re.match(self.regexp, uri))
        )

    def do(self, uri, remaining, resolution_detail, resolver_context = None, **params):
        if self.test_condition(uri, resolver_context, **params): 
            if resolution_detail is not None: 
                if self.regexp is not None:
                    resolution_detail.append("conditional: " + self.regexp)
                else:
                    resolution_detail.append("conditional: " + resolver_context)
            return self.rule.do(uri, remaining, resolution_detail, resolver_context = resolver_context, **params)
        else:
            return remaining.next().do(uri, remaining, resolution_detail, resolver_context = resolver_context, **params)


class ConditionalGroup(Conditional):
    """combines a Conditional and a Group to make a conditionally-executing Group."""
    
    def __init__(self, regexp = None, context = None, rules = []):
        Conditional.__init__(self, Group(rules = rules), regexp, context)



    
class NotFound(ResolverRule):
    """returns not found.  place at the bottom of Group and Match chains to have them terminate."""
    name = 'notfound'
    
    def __init__(self, silent = False):
        """silent = True indicates that a resulting TopLevelNotFound exception should be 'silent', i.e.
        not logged.  this is used when a 404 error is being propigated to another application layer."""
        self.silent = silent
        
    def do(self, uri, remaining, resolution_detail, **params):
        if resolution_detail is not None: resolution_detail.append("notfound")
        raise exception.ComponentNotFound("cant locate component %s" % uri, resolution_detail, silent = self.silent)
        
class PathTranslate(ResolverRule):
    """ performs path translation rules on the given uri and sends control off to the
    next rule."""
    
    name = 'pathtranslate'
    
    def __init__(self, *translations):
        
        if len(translations):
            self.path_translate = translations
        else:
            self.path_translate = None

    def modifies_uri(self):return True
        
    def do_init_resolver(self, resolver, remaining_rules, path_translate = None, **params):
        if self.path_translate is None:
            self.path_translate = path_translate
        if self.path_translate is not None and not callable(self.path_translate):
            pt = self.path_translate
            def trans(url):
                for rule in pt:
                    url = re.sub(rule[0], rule[1], url)
                return url
            self.path_translate = trans
    def do(self, uri, remaining, resolution_detail, **params):
        if self.path_translate is None:
            return remaining.next().do(uri, remaining, resolution_detail, **params)

        olduri = uri
        uri = self.path_translate(uri)
            
        if resolution_detail is not None: 
            resolution_detail.append("translate: %s -> %s" % (olduri, uri))
            
        return remaining.next().do(uri, remaining, resolution_detail, **params)

class URICache(ResolverRule):
    """caches the result of either a given nested rule, or the remaining rules in its chain, 
    based on the incoming URI."""
    
    name = 'uricache'
    
    def __init__(self, source_cache_size = None, rule = None):
        
        """
        rule is a single ResolverRule (or Group) whos results will be cached on a 
        per-uri basis.  if rule is None, then the result of remaining rules in the current chain
        will be cached based on the incoming URI.  This rule cannot be a PathTranslation
        rule or other uri-modifying rule, since the translated uri is not stored.
        
        source_cache_size is the size of the LRUCache to create.
        if source_cache_size is None, it will use the Resolver's source cache, 
        sharing it with any other URICaches that also use the Resolver's source cache.
        
        When using the Resolver's source cache, if the Resolver has use_static_source 
        disabled, then caching is disabled in this URICache.
        """
        self.source_cache_size = source_cache_size
        self.rule = rule
        
    def do_init_resolver(self, resolver, remaining_rules, **params):
        if self.rule is not None:
            if self.rule.modifies_uri():
                raise exception.ConfigurationError("can't cache single rule %s - it is a URI-modifying rule" % str(self.rule))
            self.rule.init_resolver(resolver, remaining_rules, **params)
                
        if self.source_cache_size is None:
            self.source_cache = resolver.source_cache
        else:
            self.source_cache = util.LRUCache(self.source_cache_size)
        

    def do(self, uri, remaining, resolution_detail, **params):
        if self.source_cache is None:
            if self.rule is not None:
                return self.rule.do(uri, remaining, resolution_detail, **params)
            else:
                return remaining.next().do(uri, remaining, resolution_detail, **params)
        
        # create a key that has a stringified version of the parameters plus the URI.
        # parameters are expected to be pretty much true/false, strings, and maybe lists of strings.
        key = repr(params) + "|" + uri
        
        try:
            # try to pull results from the cache
            cached = self.source_cache[key]
            
            # update detail with the cached detail
            if resolution_detail is not None: 
                resolution_detail += cached['detail']
        
            if cached.has_key('exception'):
                # single rule, which was run with an empty chain - so StopIteration
                # corresponds to just continuing on the rest of the chain.
                if self.rule is not None and isinstance(cached['exception'], StopIteration):
                    return remaining.next().do(uri, remaining, resolution_detail, **params)
                else:
                    # else it was a single rule that raised ComponentNotFound, or
                    # it was the remainder of the rule chain, so raise exception
                    raise cached['exception']
            else:
                # got a cached result - return it
                resolution = copy.copy(cached['resolution'])
                resolution.detail = resolution_detail
                return resolution
        except KeyError:
            # ok, nothing in the cache, so run the rule or rules and cache the result
            
            cached_detail = []
            cached_detail.append("cached: " + str(time.time()))

            cached = None
            
            can_cache = True
            
            try:        
                try:
                    if self.rule is not None:
                        # single rule - run it in an empty chain
                        resolution = self.rule.do(uri, iter([]), cached_detail, **params)
                    else:
                        # remaining rules - run the rest of the chain
                        resolution = remaining.next().do(uri, remaining, cached_detail, **params)
            
                    can_cache = resolution.can_cache
                    cached = {'resolution': resolution, 'detail': cached_detail}
                    
                    if resolution_detail is not None: resolution_detail += cached_detail
                    return resolution
                    
                except StopIteration, si: 
                    # normal end of chain reached
                    cached = {'exception': si, 'detail': cached_detail}
                    if self.rule is not None:
                        # single rule - continue on with the rest of the chain
                        if resolution_detail is not None: resolution_detail += cached_detail
                        return remaining.next().do(uri, remaining, resolution_detail, **params)
                    else:
                        # remaining chain - raise the StopIteration
                        if resolution_detail is not None: resolution_detail += cached_detail
                        raise si

                except exception.ComponentNotFound, cf: 
                    # a rule wants to force not found
                    cached = {'exception': cf, 'detail': cached_detail}
                    if resolution_detail is not None: resolution_detail += cached_detail
                    raise cf
            finally:
                # cache the result
                if can_cache is True:
                    self.source_cache[key] = cached
            
class ResolveDhandler(ResolverRule):
    """collects all the resolver rules below it and runs them for the requested uri, and if not found
    re-runs them again for the "dhandler" filename, pruning off path tokens until the root is reached."""
    
    name = 'resolvedhandler'
    
    def __init__(self, dhandler_name = 'dhandler'):
        self.dhandler_name = dhandler_name

    
    def do(self, uri, remaining, resolution_detail, enable_dhandler = False, declined_components = None, **params):
        if not enable_dhandler:
            return remaining.next().do(uri, remaining, resolution_detail, enable_dhandler = False, declined_components = declined_components, **params)
            

        list = [elem for elem in remaining]

        searchuri = uri
        parentdir = None
        raisecfound = False
        dhandler_path = None
        
        while True:
            iterator = iter(list)       
            try:
                if parentdir is not None:
                    dhandler_path = uri[len(parentdir):]
                resolution = iterator.next().do(searchuri, iterator, resolution_detail, dhandler_path = dhandler_path, **params)
                if resolution:
                    if declined_components and declined_components.has_key(resolution.csource.id):
                        if resolution_detail is not None: resolution_detail.append("declined")
                    else:
                        resolution.dhandler_path = dhandler_path
                        
                        return resolution
                
            # keep track on which type of exception the chain raises when it reaches the end
            # this is usually based on if a NotFound element is at the end of the chain
            except StopIteration: pass
            except exception.ComponentNotFound: raisecfound = True

            # csource not found, or found but its been declined
            # modify path to be a dhandler path, or move dhandler path up one token
            (parentdir, name) = unixpath.split(searchuri)
            if name == self.dhandler_name:
                # already looked for dhandler, and no more tokens.  not found
                if not parentdir or parentdir == '/':
                    # raise the exception that would normally have happened
                    # in the resolution chain
                    if raisecfound:
                        raise exception.ComponentNotFound("cant locate uri %s" % uri, resolution_detail)
                    else:
                        raise StopIteration

                (parentdir, token) = unixpath.split(parentdir)
            else:
                # entering "dhandler" mode, append to detail
                if resolution_detail is not None: resolution_detail.append("dhandler")

            searchuri = unixpath.join(parentdir, self.dhandler_name)

        assert(False, "enableDhandler loop failed")     

class ResolveUpwards(ResolverRule):
    """takes the rules below it and applies "search upwards" logic to them, where it iteratively breaks off
    path tokens behind the filename and searches upwards until the root path is reached.
    
    if require_context is True, then this rule only takes effect if "search_upwards" is sent within the resolution options.
    this indicates that the upwards resolve is context-sensitive, and therefore should not be cached based on a
    URI alone.  however, by default it allows its results to be cached since in practice its used 
    strictly for autohandlers, which are always searched upwards.
    """
    
    name = 'resolveupwards'
    
    def __init__(self, require_context = True):
        self.require_context = require_context

    def do(self, uri, remaining, resolution_detail, search_upwards = False, **params):
        if self.require_context and not search_upwards:
            return remaining.next().do(uri, remaining, resolution_detail, **params)

        if resolution_detail is not None:
            resolution_detail.append("resolveupwards")

        list = [elem for elem in remaining]

        searchuri = uri
        parentdir = None
        raisecfound = False

        while True:
            iterator = iter(list)
            resolution = None
            
            try:
                resolution = iterator.next().do(searchuri, iterator, resolution_detail, **params)
                if resolution:
                    if parentdir:
                        resolution.uri_truncated = uri[len(parentdir):]
                    return resolution
                
            # keep track on which type of exception the chain raises when it reaches the end
            # this is usually based on if a NotFound element is at the end of the chain
            except StopIteration: pass
            except exception.ComponentNotFound: raisecfound = True

            (parentdir, name) = unixpath.split(searchuri)
            if not parentdir or parentdir == '/':
                # raise the exception that would normally have happened
                # in the resolution chain
                if raisecfound:
                    raise exception.ComponentNotFound("cant locate uri %s" % uri, resolution_detail)
                else:
                    raise StopIteration

            (parentdir, token) = unixpath.split(parentdir)

            searchuri = unixpath.join(parentdir, name)

        assert(False, "UpwardsResolve loop failed")     


    
class Resolver(object):
    """resolves incoming URIs against a list of rules, and returns Resolution objects and/or
    raises ComponentNotFound exceptions."""
    
    def __init__(self, 
                resolver_strategy = None,
                request_resolver = None,
                use_static_source = False,
                source_cache_size = 1000,
                track_resolution_detail = True,
                debug_file = None,
                **params 
                ):

        if resolver_strategy is None:
            resolver_strategy = [
                PathTranslate(),
                ResolveDhandler(),
                URICache(),
                ResolveUpwards(),
                ResolvePathModule(),
                ResolveModule(),
                ResolveFile()
            ]

#        if request_resolver is not None:
#            resolver_strategy.insert()
            
        self.debug_file = debug_file
   
        self.use_static_source = use_static_source 
        if use_static_source:
            self.source_cache = util.LRUCache(source_cache_size)
        else:
            self.source_cache = None

        self.track_resolution_detail = track_resolution_detail
        
        for i in range(0, len(resolver_strategy)):
            if type(resolver_strategy[i]) == str:
                try:
                    resolver_strategy[i] = rulelookup[resolver_strategy[i]]()
                except KeyError:
                    raise exception.ConfigurationError("No such resolution rule '%s'" % resolver_strategy[i])

        for i in range(0, len(resolver_strategy)):
            resolver_strategy[i].init_resolver(self, resolver_strategy[i + 1:], **params)
            
        self.resolver_strategy = resolver_strategy
            
    def resolve(self, uri, raise_error = True, **params):
        iterator = iter(self.resolver_strategy)

        if self.track_resolution_detail:
            resolution_detail = []
        else:
            resolution_detail = None

        try:            
            try:
                return iterator.next().do(uri, iterator, resolution_detail, **params)
            except StopIteration:
                if raise_error:
                    raise exception.ComponentNotFound("Cant locate component %s" % uri, resolution_detail)
                else:
                    return None
            except exception.ComponentNotFound, cf:
                if raise_error:
                    raise cf
                else:
                    return None
        finally:
            if self.debug_file is not None and resolution_detail is not None: self.debug_file.write('"' + uri + '" ' + string.join(resolution_detail, ', '))
        

class FileResolver(Resolver):pass

rulelookup = {}

thismodule = sys.modules[__name__]
for obj in thismodule.__dict__.values():
    if (type(obj) == types.ClassType or type(obj) == types.TypeType) and issubclass(obj, ResolverRule) and hasattr(obj, 'name'):
        rulelookup[obj.name] = obj
        



from myghty import interp
import re, sys, StringIO, unittest

def runComponent(componentSrc, **config):
    interpreter = interp.Interpreter(**config)
    comp = interpreter.make_component(componentSrc)
    buf = StringIO.StringIO()
    interpreter.execute(comp, out_buffer=buf)
    return buf.getvalue()


# A component with "stacked filters"
# Any of the output from subcomponent "a", should be passed
# through a\'s filter, and then the main components filter.
stackedFilterSrc='''
<&a&>
<%filter>
 return "filter(%s)" % f
</%filter>
<%def a>
 hello
 <%filter>
   return "a(%s)" % f
 </%filter>
</%def>
'''
# The resulting output should contain a string
# something like "filter(a(" which indicates that the
# main filter has been applied to the output of the subcomponents
# filter.
stackedFilterCheck="filter(a("


class FilterTests(unittest.TestCase):
    """Test that two stacked filters get properly called.
    """
    auto_flush = False
    
    def testStackedFilters(self):
        output = runComponent(stackedFilterSrc,
                              auto_flush=self.auto_flush)
        output = re.sub(r'\s', '', output)
        self.failUnless(stackedFilterCheck in output,
                        "output %s does not contain %s"
                        % (repr(output), repr(stackedFilterCheck)))
                                                           

class AutoFlushFilterTests(FilterTests):
    auto_flush = True

if __name__ == '__main__':
    unittest.main()



# $Id: csource.py 2133 2006-09-06 18:52:56Z dairiki $
# csource.py - component source objects for Myghty
# Copyright (C) 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz. 
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#

import posixpath as unixpath
import time, re, os, stat, types, sys, string
import __builtin__ as builtin
import myghty.util as util
import myghty.importer as importer
import myghty.exception as exception

class ComponentSource(object):
    """a time-sensitive descriptor object for a Myghty component."""
    def __init__(self, id, last_modified = None ):
    
        "unique identifier of the component."
        self.id = id
        
        "for file components, the filename without path."
        self.name = ""
        
        "for file components, relative directory where the component lives."
        self.dir_name = ""
        
        "for file components, this is dir_name + name"
        self.path = ""
        
        "for file components, this is the real filesystem path of the component."
        self.file_path = ""
        
        "for file components, id of the path, i.e. the key in the component_root hash."
        self.path_id = ""
        
        "the module containing the actual code for the component."
        self.module = None
        
        "guesstimate of the size of the component. this is filled in by Interpreter."
        self.modulesize = 0
                
        if last_modified:
            self.last_modified = last_modified
        else:
            self.last_modified = int(time.time())

    def can_compile(self):return True
    
    def get_component_source_file(self):raise NotImplementedError()
    
    def get_component_source(self):raise NotImplementedError()
    
    def get_object_code(self, compiler, file):
        """compiles the source of this component using the given compiler,
        sending output to the given file"""
        compiler.compile(
            source = self.get_component_source(),
            name = self.id,
            file = file,
            input_file = self.file_path,
        )
        
class ModuleComponentSourceSingleton(type):
    """a metaclass for ModuleComponentSource which allows its constructor to cache the 
    inspection results of the "arg" constructor parameter. """
    def __call__(self, **kwargs):
        arg = kwargs.get('arg', None)
        arg_cache = kwargs.pop('arg_cache', None)
        use_static_source = kwargs.pop('use_static_source', False)
        if arg_cache is not None and arg is not None:
            try:
                mc = arg_cache[arg]
            except KeyError:
                mc = type.__call__(self, **kwargs)
                arg_cache[arg] = mc
            return mc.copy(use_static_source=use_static_source)
        else:
            return type.__call__(self, **kwargs)
       
class ModuleComponentSource(ComponentSource):
    """represents a loadable module containing either a ModuleComponent class, a function, a class
    instance which is callable or a method on a class instance. """
    
    __metaclass__ = ModuleComponentSourceSingleton
    
    def __init__(self, 
        arg = None,
        module = None, 
        objpath = None,
        name = None,
        callable_ = None,
        class_ = None,
        last_modified = None):

        if arg is not None:
            if isinstance(arg, str):
                (modname, objname) = arg.split(':')
                module = importer.module(modname)
                objpath = objname.split('.')

            if objpath is not None:
                arg = module
                for t in objpath:
                    arg = getattr(arg, t)

            name = self.inspect_target(arg, objpath)

            if module is None:
                if self.class_ is not None:
                    module = sys.modules[self.class_.__module__]
                elif self.has_method:
                    module = importer.module(self.callable_.im_class.__module__)
                else:
                    module = importer.module(self.callable_.__module__)

        else:
            self.class_ = class_
            self.callable_ = callable_
            self.has_method = callable_ is not None and (isinstance(arg, types.MethodType) or not isinstance(arg, types.FunctionType))
            
        if last_modified is None:
            last_modified = importer.mod_time(module)

        ComponentSource.__init__(self, "module|%s:%s" % (module.__name__, name), last_modified = last_modified)
            
        self.module = module
        self.objpath = objpath
        self.name = name


    def reload(self, module):
        # the "objpath" is a list of tokens that allow us to traverse from the
        # module to the callable/class unit.  if we dont have that, then we can't 
        # reload dynamically.
        if self.objpath is None:
            return

        self.module = module        
        arg = module
        for t in self.objpath:
            arg = getattr(arg, t)

        self.inspect_target(arg, self.objpath)

    def inspect_target(self, arg, objpath):
        self.has_method = False

        self.class_ = None
        self.callable_ = None
        
        if isinstance(arg, types.TypeType) or isinstance(arg, types.ClassType):
            self.class_ = arg
            name = "class:" + arg.__name__
        elif isinstance(arg, types.MethodType):
            self.callable_ = arg
            if objpath is not None:
                name = "method:" + string.join(objpath, '_')
            else:
                name = "method:" + arg.im_func.__name__
            self.has_method = True
        elif isinstance(arg, types.FunctionType):
            self.callable_ = arg
            if objpath is not None:
                name = "function:" + string.join(objpath, '_')
            else:
                name = "function:" + arg.__name__
        elif callable(arg):
            arg = arg.__call__
            self.callable_ = arg
            if objpath is not None:
                name = "callable:" + string.join(objpath, '_')
            else:
                name = "callable:" + arg.__class__.__name__
            self.has_method = True
        else:
            raise "arg is " + repr(arg)
            
        return name
    
    def copy(self, use_static_source = False): 
        if use_static_source:
            last_modified = self.last_modified
        else:
            last_modified = importer.modulemodtime(self.module)
            
        return ModuleComponentSource(module = self.module, objpath = self.objpath, callable_ = self.callable_, name=self.name, class_ = self.class_, last_modified = last_modified)
        
    def can_compile(self):
        return False


class MemoryComponentSource(ComponentSource):
    def __init__(self, source, id = None,  last_modified = None):
        if id is None:
            id = str(builtin.id(self))
            
        ComponentSource.__init__(self, id, last_modified)
        self.source = source
            
    def get_component_source_file(self):
        return util.StringIO(self.source)
    
    def get_component_source(self):
        return self.source

    
class FileComponentSource(ComponentSource):
    def __init__(self, id, path_id, path, file_path, last_modified=None):
        if last_modified is None:
            last_modified = os.stat(file_path)[stat.ST_MTIME]
            
        ComponentSource.__init__(self, id, last_modified)
        self.file_path = file_path
        self.path = path
        self.path_id = path_id
        (self.dir_name, self.name) = unixpath.split(path)
    
    def get_component_source_file(self):
        return open(self.file_path)
    
    def get_component_source(self):
        return self.get_component_source_file().read()



# $Id: compiler.py 2133 2006-09-06 18:52:56Z dairiki $
# compiler.py - compiles parsed files into a parse tree for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz.
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#


"""
 Middle tier between a Lexer and an ObjectGenerator.  receives parse events from Lexer
and constructs a parse tree, then calls an ObjectGenerator to generate an object file from 
the parse tree.  the parse tree structure is language-neutral and contains no python code.

Compiler is currently stateful and is not thread-safe. the clone() method can be used
to create copies of this object for use in multiple threads.
"""


from myghty.util import *
from myghty import exception
import myghty.lexer
import myghty.objgen as objgen
import myghty.args as args
import string, re, sys, warnings



# blocks that can only be in the top level
top_level_only_block = dict(map (lambda x:[x ,True], ['once', 'shared', 'threadonce']))

# valid flags in use
valid_comp_flag = dict(map (lambda x:[x ,True], ['inherit']))

# map of block names pointing to compile method names
BLOCKS = {
    'python'    : 'python_block',
    }


argscopes = {
    'request' : args.RequestArg,
    'subrequest' : args.SubRequestArg,
    'dynamic' : args.DynamicArg,
    'component' : args.LocalArg
}

class Compiler:

    # compiled component modules must have this defined as their
    # _MAGIC_NUMBER or an IncompatibleCompiler exception will be raised
    MAGIC_NUMBER = 5
    
    class BlockFlyweight:
        """an object that represents
         a construct in the original source file.  maintains a reference to a list
         of source file items and uses a context to act upon the list entry
         in question, so that very few objects need to be created """
        def __init__(self):pass

        def get_line(self, context):
            return context.entry[2]
            
        def get_line_number(self, context):
            return context.entry[1]
            
        def append(self, list, line, linenumber):
            list.append([self, linenumber, line])
            
        def accept_visitor(self, visitor, context):raise NotImplementedError

    class CodeBlock(BlockFlyweight):
        """represents a <%python> section"""
        def get_lines(self, context):
            block = context.entry[2]
            lines = re.split(r"\n", block)
            return lines
            
        def accept_visitor(self, visitor, context):
            visitor.visit_code_block(self, context)

    class Substitution(BlockFlyweight):
        """represents a <% %> line """
        def accept_visitor(self, visitor, context):
            visitor.visit_substitution(self, context)

        def append(self, list, line, escape, linenumber):
            list.append([self, linenumber, line, escape])
            
        def get_escapes(self, context):
            return context.entry[3]
    
    class TextLine(BlockFlyweight):
        """ represents a line of plain text"""
        def accept_visitor(self, visitor, context):
            visitor.visit_text_line(self, context)

    class CodeSingleLine(BlockFlyweight):
        """ represents a % line """
        def accept_visitor(self, visitor, context):
            visitor.visit_code_single_line(self, context)

    class ComponentCall(BlockFlyweight):
        """ represents a component call without content <& &>"""
        def append(self, list, component, args, linenumber):
            list.append([self, linenumber, component, args])
        
        def get_component(self, context):
            return context.entry[2]
        
        def get_args(self, context):
            return context.entry[3]
            
        def accept_visitor(self, visitor, context):
            visitor.visit_component_call(self, context)

    class ComponentContentCall(ComponentCall):
        """ represents a component call with content <&| &>"""
        def accept_visitor(self, visitor, context):
            visitor.visit_component_content_call(self, context)

    class ComponentContentCallEnd(ComponentCall):
        """ represents a component call with content end tag </&>"""
        def accept_visitor(self, visitor, context):
            visitor.visit_component_content_call_end(self, context)

    class Closure(BlockFlyweight):
        def append(self, list, compiled, linenumber):
            list.append([self, linenumber, compiled])

        def get_compiled(self, context):
            return context.entry[2]
            
        def accept_visitor(self, visitor, context):
            visitor.visit_closure_block(self, context)
            
    class BlockFlyweightIterator:
        """an iterator that loops through an array of BlockFlyweight references and
        their associated data,
        and acts as the context to send to their get() methods"""
        def __init__(self, list):
            self.list = list
            self.index = -1
            self.entry = None
            
        def __iter__(self):
            return self
            
        def next(self):
            self.index += 1
            if self.index >= len(self.list): raise StopIteration()
            self.entry = self.list[self.index]
            return self.entry[0]

    # singleton instances of BlockFlyweights
    codeblock = CodeBlock()
    textline = TextLine()
    codesingleline = CodeSingleLine()
    substituteline = Substitution()
    componentcall = ComponentCall()
    componentcontentcall = ComponentContentCall()
    componentcontentcallend = ComponentContentCallEnd()
    closure = Closure()
    
    def __init__(self, 
            python_pre_processor = None, 
            python_post_processor = None, 
            text_post_processor = None, 
            lexer = None, 
            generator = None,
            default_escape_flags = [],
            use_source_line_numbers = True,
            allow_globals = [],
            disable_unicode = False,
            **params):
            
        self.current_compile = None
        if lexer:
            self.lexer = lexer
        else:
            self.lexer = myghty.lexer.Lexer(**params)
            
        if generator:
            self.generator = generator
        else:
            self.generator = objgen.PythonGenerator()
            
        self.default_escape_flags = default_escape_flags
        self.python_pre_processor = python_pre_processor
        self.python_post_processor = python_post_processor
        self.text_post_processor = text_post_processor
        self.use_source_line_numbers = use_source_line_numbers
        self.allow_globals = allow_globals
        self.disable_unicode = bool(disable_unicode)
        
    def get_object_id(self):
        """identification string placed at the top of compiled files"""
        return "%s|%s|%s" % (self.lexer.get_object_id(),
                             "Myghty.Compiler",
                             repr(self.get_magic_number()))

    def get_magic_number(self):
        return Compiler.MAGIC_NUMBER, {'disable_unicode': self.disable_unicode}

    def clone(self, **params):
        """creates a clone of this Compiler.  allow the Prototype pattern
        to be used in creating compilers for use in other threads."""

        params['lexer'] = self.lexer.clone()
        params['generator'] = self.generator.clone()
        clone = ConstructorClone(self, **params)
        return clone.clone()



    
    class Compiled:
        "Stores information about the currently compiling block"
        def __init__(self, compiler, fileid, name, in_main = True, parent = None, block_type = None, flags = None):
            self.compiler = compiler
            self.lexer = compiler.lexer
            self.parent = parent
            self.in_main = in_main
            self.block_type = block_type
            
            self.fileid = fileid
            self.name = name    
                
            self.component_content_call_stack = []
            
            self.in_block = None

            self.args = OrderedDict()
            
            if flags is not None:
                self.flags = OrderedDict([flags])
            else:
                self.flags = OrderedDict()
            
            self.attr = OrderedDict()
            
            self.named_blocks = {'def': {}, 'method': {}}
            
            self.blocks = {
                    'cleanup' : [],
                    'filter' : [],
                    'init' : [],
                    'once' : [],
                    'threadonce' : [],
                    'shared' : [],
                    'body' : []
                }

            self.start_blocks = {
                'args' : -1,
                'flags' : -1,
                'attr': - 1,
            }

            self.encoding = None
            

        def accept_visitor(self, visitor):
            if self.in_main:
                visitor.visit_component_block(self)
            elif self.block_type == 'def':
                visitor.visit_def_block(self)
            elif self.block_type == 'method':
                visitor.visit_method_block(self)
                
        def add_argument(self, name, default, linenumber, scope):
            if self.args.has_key(name):
                self.lexer.raise_syntax_error("component argument %s already defined" % name)

            try:
                self.args[name] = argscopes[scope or 'component'](name, default = default, linenumber = linenumber)
            except KeyError:
                self.lexer.raise_syntax_error("Unknown %%args scope '%s'" % scope)


        def has_named_block(self, block_type, name):
            if block_type is None:
                for value in self.named_blocks.values():
                    if value.has_key(name):
                        return True
                else:
                    return False
            else:
                return self.named_blocks.has_key(block_type) and self.named_blocks[block_type].has_key(name)
            
        def get_named_block(self, block_type, name):
            if block_type is None:
                for value in self.named_blocks.values():
                    if value.has_key(name):
                        return value[name]
                else:
                    raise KeyError(name)
            else:
                return self.named_blocks[block_type][name]
            
        def add_named_block(self, block_type, name, block):
            self.named_blocks[block_type][name] = block

        def get_named_blocks(self, block_type = None):
            if block_type is not None:
                return self.named_blocks[block_type].values()
            else:
                list = []
                for d in self.named_blocks.values():
                    list += d.values()
                return list

        def get_body(self):
            return self.blocks['body']
            
        def get_block_list(self, block_type):
            return self.blocks[block_type]

        def block_has_code(self, block_type):
            return len(self.blocks[block_type]) > 0
            
        def get_block_iterator(self, block_type):
            block = self.get_block_list(block_type)
            return Compiler.BlockFlyweightIterator(block)

        def set_encoding(self, encoding):
            if self.parent is not None:
                self.parent.set_encoding(encoding, ignore_if_already_set)
            elif self.encoding:
                self.lexer.raise_syntax_error(
                    'multiple file encoding specifiers')
            else:
                self.encoding = encoding

        def get_encoding(self):
            if self.parent is not None:
                return self.parent.get_encoding()
            return self.encoding
        
        def __str__(self):
            return self._to_string()
            
        def _to_string(self, indent = ''):
            """dumps out the structure attempting to recreate the original source file,
            more or less."""
            
            list = []
            if self.encoding is not None:
                list.append("# -*- encoding: %s -*-" % self.encoding)

            list.append("<%args>")
            for arg in self.args.keys():
                list.append(arg + " = " + repr(self.args[arg]))
            list.append("</%args>")
            
            for key in ('once', 'init', 'shared', 'body', 'cleanup'):
                block = self.get_block_list(key)
                
                if key != 'body':
                    list.append("<%%%s>" % key)
                else:
                    list.append("# body code begin")
                    
                iter = Compiler.BlockFlyweightIterator(block)
                for code in iter:
                    list.append(code.get_line(iter))

                if key != 'body':
                    list.append("</%%%s>\n" % key)
                else:
                    list.append("# body code end\n")

            
            for blockname, blocks in self.named_blocks.iteritems():
                for key, value in self.def_blocks.iteritems():
                    ind = indent + "\t"
                    list.append(ind + "<%%%s %s>" % (blockname, key))
                    list.append(value._to_string(ind + "\t"))
                    list.append(ind + "</%%%s %s>" % (blockname, key))

            
            return indent + string.join(list, "\n" + indent)
    
        
    
    def compile(self, source, name, file, input_file = None):
        """compiles a source file.
        
        source - a string representing the source of the file
        name - a name to give to the compiled object
        file - a file object that the compiled output will be streamed to."""
        
        self.current_compile = Compiler.Compiled(compiler = self, fileid = name, name = 'top_level')

        # Preprocess the source.  The preprocessor routine is handed a
        # reference to the entire source.
        if self.python_pre_processor:
            try:
                source = self.python_pre_processor(source)
            except Exception, e:
                raise exception.Compiler(e.args)
    
        self.lexer.lex(source, name, self, input_file)
        self.compiled_component(file)
        


    def start_block(self, block_type, attributes = None):
        """called by Lexer to indicate a <%block> tag """
                
        compile = self.current_compile

        if top_level_only_block.has_key(block_type) and not compile.in_main:
            self.lexer.raise_syntax_error("Cannot define a %s section inside a method or subcomponent" % block_type)

        if compile.start_blocks.has_key(block_type):
            compile.start_blocks[block_type] = self.lexer.line_number()
        
        if compile.in_block:
            self.lexer.raise_syntax_error("Cannot nest a %s inside a %s block" % (block_type, c.in_block))

        compile.in_block = block_type


    def end_block(self, block_type):
        """called by Lexer to indicate a </%block> tag"""
        compile = self.current_compile
        
        if compile.in_block != block_type:
            self.lexer.raise_syntax_error("End of %s encountered while in %s block" % (block_type, compile.in_block))
        
        compile.in_block = None

    
    def text_block(self, block, **params):
        Compiler.textline.append(self.current_compile.get_block_list('body'), block, self.lexer.line_number())

        
    # comment - discard
    def doc_block(self, **params):pass

    def magic_encoding_comment(self, encoding, **params):
        """A magic -*- encoding: foo -*- style comment."""
        assert encoding and type(encoding) is str
        self.current_compile.set_encoding(encoding)
        
    def post_process_text(self, code):
        if self.text_post_processor:
            try:
                return self.text_post_processor(code)
            except Exception, e:
                raise exception.Compiler(e.args)
        else:
            return code
        
    
    def post_process_python(self, code):
        if self.python_post_processor:
            try:
                return self.python_post_processor(code)
            except Exception, e:
                raise exception.Compiler(e.args)
        else:
            return code 
    
    def raw_block(self, **params):
        # see if we have a method corresponding to this block and call it if so

        if BLOCKS.has_key(params['block_type']):
            method = BLOCKS[params['block_type']]
            
            # call method dynamically
            return getattr(self, method)(**params)

        else:
            Compiler.codeblock.append(self.current_compile.get_block_list(params['block_type']), params['block'], self.lexer.line_number())

        
    def variable_declaration(self, block_type, name, default, scope):
        
        """Inserts a variable declaration from the C<< <%args> >> section into
        the component."""

        if block_type != 'args':
            self.lexer.raise_syntax_error("Variable Declaration called inside a %s block" % block_type)
        
        self.current_compile.add_argument(name, default, self.lexer.line_number(), scope)
        
    def key_value_pair(self, block_type, key, value):
        try :
            dict = {"flags": self.current_compile.flags, "attr": self.current_compile.attr}[block_type]
            if dict.has_key(key):
                self.lexer.raise_syntax_error("%s %s already defined" % (key, block_type))
                
            dict[key] = value

            if block_type == "flags" and key == "encoding":
                #warnings.warn(
                #    "The 'encoding' flag has been deprecated, "
                #    "use an encoding magic comment instead.",
                #    category=DeprecationWarning)
                self.current_compile.set_encoding(eval(value, {}))

        except KeyError:
            exception.Compiler("key_value_pair called inside a %s block" % block_type)
        

    def python_line(self, line):
        Compiler.codesingleline.append(self.current_compile.get_body(), line, self.lexer.line_number())
        
    def python_block(self, block, **params):
        Compiler.codeblock.append(self.current_compile.get_body(), block, self.lexer.line_number())

    def substitution(self, line, escape):
        if escape or len(self.default_escape_flags):
            dict = OrderedDict()

            if escape:
                escapes = re.split(r"\s*,\s*", escape)
                for esc in escapes:
                    dict[esc] = 1

            if self.default_escape_flags and not dict.has_key('n'):
                for esc in self.default_escape_flags:
                    dict[esc] = 1

            if dict.has_key('n'): del dict['n']

            escapes = dict.keys()
        else:
            escapes = None
                
        Compiler.substituteline.append(self.current_compile.get_body(), line, escapes, self.lexer.line_number())

    def start_component(self):
        pass

    def end_component(self):
        compile = self.current_compile
        if len(compile.component_content_call_stack) > 0:
            self.lexer.raise_syntax_error("Not enough component-with-content ending tags found")
    


    def compiled_component(self, file):
        self.generator.generate(self, self.current_compile, file)
        

    def start_named_block(self, block_type, name, attributes):
        compile = self.current_compile
        
        # Error if defining one def or method inside another
        if block_type != 'closure' and not compile.in_main:
            self.lexer.raise_syntax_error("Cannot define a %s block inside a method or subcomponent" % block_type)

        if re.search(r"[^.\w-]", name):
            self.lexer.raise_syntax_error("Invalid %s name: %s" % (block_type, name))


        # error if we have a named block with this name already
        if block_type != 'closure' and compile.has_named_block(None, name):
            block = compile.get_named_block(None, name)
            self.lexer.raise_syntax_error("%s block %s already exists" % (block.block_type, name))
                
        # make a new compile object, set it to be our current
        newcompile = Compiler.Compiled(compiler = self, parent = self.current_compile, in_main = False, fileid = self.current_compile.fileid, name = name, block_type = block_type, flags = attributes)
        if block_type != 'closure':
            compile.add_named_block(block_type, name, newcompile)
        else:
            Compiler.closure.append(self.current_compile.get_body(), newcompile, self.lexer.line_number())
        
        self.current_compile = newcompile


    def end_named_block(self, block_type):
        # reset current compile object to be its parent
        if self.current_compile.parent:
            self.current_compile = self.current_compile.parent
        else:
            raise exception.Compiler("end_named_block found no parent block")


    def _generic_component_call(self, call, iscontent = False, isclose = False):
    
        if isclose:
            try:
                call = self.current_compile.component_content_call_stack.pop()
            except IndexError:
                self.lexer.raise_syntax_error("found component with content ending tag but no beginning tag")
        else:
            call = string.strip(call)

        (component, args) = (re.split(r",", call, 1) + [''])[0:2]
        component = string.strip(component)
        args = string.strip(args)

        if not iscontent:       
            Compiler.componentcall.append(self.current_compile.get_body(), component, args, self.lexer.line_number())
        else:
            if not isclose:
                self.current_compile.component_content_call_stack.append(call)
                Compiler.componentcontentcall.append(self.current_compile.get_body(), component, args, self.lexer.line_number())
            else:
                Compiler.componentcontentcallend.append(self.current_compile.get_body(), component, args, self.lexer.line_number())

    def component_call(self, call):
        self._generic_component_call(call, False, False)
        
    def component_content_call(self, call):
        self._generic_component_call(call, True, False)
    
    def component_content_call_end(self):
        self._generic_component_call(None, True, True)


    def get_encoding(self):
        return self.current_compile.get_encoding()
        



# test/reload.py
"""Test that components reload properly when their source is touched.
"""
import unittest, os, shutil, stat, sys, time, warnings
from StringIO import StringIO

import testbase
from myghty.interp import Interpreter
from myghty.http.WSGIHandler import WSGIHandler
from myghty.resolver import NotFound
import myghty.importer

class ModuleRunner:
    """A minimal context for executing components.
    """
    def __init__(self, **params):
        self.interpreter = Interpreter(**params)

    def executeComponent(self, comp):
        buf = StringIO()
        self.interpreter.execute(comp, out_buffer=buf)
        return buf.getvalue()

class WSGIModuleRunner:
    """An WSGI context for executing components.

    We need this to test Routes components, since they require
    a request (r) object.
    """
    
    def __init__(self, **params):
        self.app = WSGIHandler(**params).handle
        
    def executeComponent(self, comp):
        environ = {
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'PATH_INFO': comp,
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': 8000,
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'HTTP_HOST': 'localhost:8000',
            'wsgi.version': (1,0),
            'wsgi.url_scheme': 'http',
            'wsgi.input': StringIO(),
            'wsgi.errors': sys.stderr,
            'wsgi.multithread': False,
            'wsgi.multiprocess': False,
            'wsgi.run_once': False,
            }

        buf = StringIO()
        def start_response(status, response_headers):
            return buf.write
            
        for s in self.app(environ, start_response):
            buf.write(s)
            
        return buf.getvalue()
    
def purge_module(module_name):
    try:
        module = sys.modules[module_name]
    except KeyError:
        return

    base, ext = os.path.splitext(module.__file__)
    for ext in '.pyc', '.pyo':
        try:
            os.unlink(base + ext)
        except OSError:
            pass

    del sys.modules[module_name]
    
class ComponentReloadTests:
    """Abstract base class of tests for component reloading."""

    module_runner_class = ModuleRunner
    
    def setUp(self):
        # Blow away cache
        shutil.rmtree(os.path.join(self.cache, 'obj'),
                      ignore_errors=True)
        # Clear importer cache
        myghty.importer.modules.clear()

        self.createComponentSource()

        params = self.resolverArgs()
        self.module_runner = self.module_runner_class(data_dir=self.cache,
                                                      **params)


    def tearDown(self):
        self.removeComponentSource()

    def srcFile(self):
        """The full path the the module source file.
        """
        raise RuntimeError, "pure virtual"

    def srcText(self, text="howdy"):
        """Get the contents of the module source.

        The module, when run, will output the value of text.
        return text
        """
        raise RuntimeError, "pure virtual"

    def resolverArgs(self):
        """Get the resolver arguments to pass to the Myghty interperter.
        """
        raise RuntimeError, "pure virtual"

    def componentPath(self):
        """Get the (URL) path to the component.
        """
        raise RuntimeError, "pure virtual"

    def createComponentSource(self, text="howdy"):
        file(self.srcFile(), "w").write(self.srcText(text))

    def changeComponentSource(self, text="changed"):
        """Covertly changes the component source without changing mtime.
        """
        srcfile = self.srcFile()
        mtime = os.stat(srcfile)[stat.ST_MTIME]
        self.createComponentSource(text)
        os.utime(srcfile, (mtime, mtime))

    def removeComponentSource(self):
        os.unlink(self.srcFile())
        base, ext = os.path.splitext(self.srcFile())
        if ext == '.py':
            for ext in '.pyc', '.pyo':
                try:
                    os.unlink(base + ext)
                except OSError:
                    pass

    def executeComponent(self):
        return self.module_runner.executeComponent(self.componentPath())

    def testComponent(self, text="howdy"):
        """Test that we can resolve and execute our component.
        """
        self.failUnlessEqual(self.executeComponent(), text)

    def testReload(self):
        """Test that component gets reloaded when it is modified.
        """
        self.testComponent()
        time.sleep(1)
        self.removeComponentSource()
        self.createComponentSource(text="changed")
        self.testComponent("changed")

    def testCompileCache(self):
        """Test that component doesn't get reloaded when it is not modified.

        (Actually, tests that component doesn't get reloaded when the source
        is modified, but the source's mtime remains unchanged.)
        """
        self.testComponent()
        time.sleep(1)
        self.changeComponentSource(text="changed")
        self.testComponent()

################################################################
#
# (The actual unittest.TestCase sub-classes begin here.)
#
################################################################


class FileComponentReloadTests(ComponentReloadTests, testbase.MyghtyTest):
    
    def srcFile(self):
        return os.path.join(self.htdocs, "comp.myt")

    def srcText(self, text="howdy"):
        return text

    def resolverArgs(self):
        return {'component_root': self.htdocs}

    def componentPath(self):
        return "/comp.myt"
    
    
class ModuleComponentReloadTests(ComponentReloadTests, testbase.MyghtyTest):
    def setUp(self):
        purge_module('module')
        ComponentReloadTests.setUp(self)
        
    def srcFile(self):
        return os.path.join(self.lib, "module.py")

    def srcText(self, text="howdy"):
        return "def comp(m): m.write('%s')" % text

    def resolverArgs(self):
        try:
            del sys.modules['module']
        except KeyError:
            pass
        return {'module_components': [{'/mycomp': 'module:comp'}]}
    
    def componentPath(self):
        return "/mycomp"

class ModuleRootComponentReloadTests(ComponentReloadTests,
                                     testbase.MyghtyTest):
    def setUp(self):
        purge_module('modroot')
        ComponentReloadTests.setUp(self)

    def srcFile(self):
        return os.path.join(self.lib, "modroot.py")

    def srcText(self, text="howdy"):
        return """
class Top:
    def comp(self, m): m.write('%s')
top = Top()
""" % text

    def resolverArgs(self):
        try:
            del sys.modules['modroot']
            pass
        except KeyError:
            pass
        return {'module_root': ['modroot']}
    
    def componentPath(self):
        return "top/comp"

class WSGIModuleRootComponentReloadTests(ModuleRootComponentReloadTests):
    """Run the module_root test in a WSGI context.

    This exercised a buglet in myghty.http.HTTPHandler.py.
    """
    module_runner_class = WSGIModuleRunner



os.environ['MYGHTY_ENV'] = 'debug'   # Otherwise RoutesResolver pukes
try:
    from routes.base import Mapper

except ImportError:
    # Routes appears not to be installed
    class RoutesComponentReloadTests(unittest.TestCase):
        def testRoutesMissing(self):
            warnings.warn("Cannot import routes.base.  Skipping Routes tests.")

else:
    # Routes seems to be installed
    from myghty.ext.routeresolver import RoutesResolver

    class RoutesComponentReloadTests(ComponentReloadTests,
                                     testbase.MyghtyTest):
        module_runner_class = WSGIModuleRunner
    
        def srcFile(self):
            return os.path.join(self.lib, "routes_controller.py")

        def srcText(self, text="howdy"):
            return """
class RoutesController:
    def index(self, m, r):
        m.write('%s')
routes = RoutesController()
""" % text

        def resolverArgs(self):
            mapper = Mapper()
            mapper.connect(':controller/:action')

            routes_resolver = RoutesResolver(
                mapper=mapper,
                controller_root = self.lib)
            
            return {'resolver_strategy': [routes_resolver, NotFound()]}
    
        def componentPath(self):
            return "/routes"

class AnonymousFunctionComponentReloadTests(testbase.ComponentTester):
    def test(self):
        def mycomp(m):
            return 1
        comp1 = self.makeModuleComponent(mycomp)

        def mycomp(m):
            return 2
        comp2 = self.makeModuleComponent(mycomp)

        print comp1.component_source.id
        self.failIfEqual(comp1, comp2, "(Currently broken, expect failure.)")
        self.failUnlessEqual(self.runComponent(comp1), 1)
        self.failUnlessEqual(self.runComponent(comp2), 2)
        
if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)



# $Id: component.py 2133 2006-09-06 18:52:56Z dairiki $
# component.py - component base classes for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz. 
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

"""the component package defines the Component class and several subclasses.  FileComponent
serves as the base class for template style-components, and ModuleComponent the base
for module-based components."""

from myghty import exception
from myghty.util import *
from myghty.args import *
import myghty.request
import myghty.csource

import weakref, string
import posixpath as unixpath
import sys, inspect, types

__all__ = ['Component', 'ModuleComponent', 'HTTPModuleComponent', 'FileComponent', 'SubComponent']

argtypes = (            
    (RequestArg, True, 'required_request_args'),
    (RequestArg, False, 'request_args'),
    (SubRequestArg, True, 'required_subrequest_args'),
    (SubRequestArg, False, 'subrequest_args'),
    (LocalArg, True, 'required_args'),
    (LocalArg, False, 'args'),
    (DynamicArg, True, 'required_dynamic_args'),
    (DynamicArg, False, 'dynamic_args'),
)
        
class Component(object):
    """Base class for all components.  the primary method is the run() method which
    executes the Component in the context of a particular request.  A Component instance
    is usually cached in memory based on its component ID and can execute many requests
    simultaneously."""
    
    def __init__(self, interpreter, component_source, **params):
        """initializes a new Component. this method is called by the Interpreter."""
        self.filter = None
        self.owner = self.parent_component = None
        
        self.flags = {}
        self.attr = {}
        self.arguments = []

        self.component_source = component_source
        
        self.is_method = self.is_file = self.is_subcomponent = self.is_module = False
        
        self.size = 0
        
        self.use_count = 0
        self.interpreter = interpreter
        self.threads_init = ThreadLocal()

    id = property(lambda self: self.component_source.id, 
        doc="unique identifier for this component")
        
    source_id = property(lambda self: self.component_source.id, 
        doc="unique identifier for the ComponentSource of this component")
        
    name = property(lambda self: self.component_source.name,
        doc="name of this component")
    
    dir_name = property(lambda self: self.component_source.dir_name,
        doc="relative directory name of the component")

    path = property(lambda self: self.component_source.path,
        doc="relative full path of the component")

    file = property(lambda self: self.component_source.file_path,
        doc="the actual filename of the component, if one exists")
        
    def _parentattr(self):
        if self.parent_component is not None:
                return self.parent_component.attributes
        else:
            return None
        
    attributes = property(lambda self: InheritedDict(self.attr, lambda: self._parentattr()))

    def component_init(self):
        """initializes the component after construction.  Calls the do_component_init() method
        before setting up the component's arguments."""
        self.do_component_init()
        self._convert_component_args()
        
    def _convert_component_args(self):
        # the args system has been overhauled to be just a list of 
        # ComponentArgs objects, which come straight from the compiled
        # component file.
        # but try to do some compatibility, for older compiled components
        # and module components that have all the separate ***_args arrays 
        # attached to them.  we generate the _args if it doesnt exist,
        # and then recreate the ***_args arrays in all cases.
        for argtype in argtypes:
            self.arguments += [argtype[0](arg, argtype[1]) for arg in getattr(self, argtype[2], [])]
            # TODO: try to use property() for this
            setattr(self, argtype[2], [arg.name for arg in self.arguments if arg.__class__ == argtype[0] and arg.required == argtype[1]])

    def do_component_init(self):
        """overridden by subclasses to perform per-instance initialization steps."""
        raise NotImplementedError()
        
    def run(self, request, **params):
        """runs this component in the context of the given request.  **params are the arguments
        sent to the component, which become ARGS in the context of the component's
        runtime environment."""
        self.use_count += 1
        compparams = {}

        for arg in self.arguments:
            arg.get_arg(request, compparams, **params)

        compparams.update(request.global_args)
    
        if not self.threads_init.exists():
            self.do_thread_init()
            self.threads_init.assign(True)

        return self.do_run_component(m = request, ARGS = params, **compparams)
        
    def has_filter(self):
        """returns True if this component defines a <%filter> section."""
        return self.filter != None
    
    def _init_filter_func(self, d = None):
        if self.flags.has_key('trim'):
            # the 'trim' flag implies that the component is
            # buffered as well
            self.flags['autoflush'] = False

            try:
                func = {'left':string.lstrip, 'right':string.rstrip, 'both':string.strip}[self.flags['trim']]
                if d is not None:
                    return lambda f, **kwargs: d(func(f), **kwargs)
                else:
                    return lambda f, **kwargs: func(f)
            except KeyError, e:
                raise exception.Interpreter("invalid 'trim' argument '%s'" % e)

        elif d is not None:
            return d

    def get_flag(self, key, inherit = False):
        """returns a flag defines in this component's <%flags> section."""
        if self.flags.has_key(key):
            return self.flags[key]
        elif inherit:
            parent = self.parent_component
            if parent is not None:
                return parent.get_flag(key, True)
        
        return None

    def get_sub_component(self, name):
        """returns a SubComponent of the given name, represented within the source
        of this Component."""
        return None
    
    def do_run_component(self, m, ARGS, **params):
        """overridden by subclasses to provide the main execution body of the component."""
        raise NotImplementedError()

    def do_thread_init(self):
        """overridden by subclasses to provide a per-thread initialization routine."""
        pass
    
    def locate_inherited_method(self, method_name):
        """returns a method SubComponent of the given name, represented either within the
        source of this Component or within the source of an inherited Component, 
        i.e. the inheritance hierarchy will be searched for the approrpriate method call."""
        raise NotImplementedError()

    def call_method(self, method_name, **params):
        """calls a method SubComponent of the given name on this component.  
        This amounts to locating it via locate_inherited_method, retrieving the
        current request via request.instance(), and then calling execute_component
        on that request.

        See also m.comp("component:methodname")
        """
        
        method_component = self.locate_inherited_method(method_name)            
        return myghty.request.instance().execute_component(method_component, args = params, base_component = self)

    def scall_method(self, method_name, **params):
        """same as call_method, but returns the component output as a string
        See also m.scomp("component:methodname")
        """
        
        method_component = self.locate_inherited_method(method_name)            
        buffer = StringIO()
        myghty.request.instance().execute_component(method_component, args = params, base_component = self, store = buffer)
        return buffer.getvalue()
    
    def use_auto_flush(self):
        """returns True if this component defines the "autoflush" flag as True, or
        if an inherited component defines this flag."""
        return self.get_flag('autoflush', inherit = True)
    

    # these are all deprecated
    def get_attribute(self, key, inherit = True): 
        return self.attributes(key)
    def set_attribute(self, key, value): 
        self.attributes(key, value)
    def get_attributes(self): 
        return self.attributes
    def attribute_exists(self, key): 
        return self.attributes.has_key(key)

    def get_owner(self):
        return self.owner
    def get_parent_component(self):
        return self.parent_component
    def get_id(self):
        return self.id
    def get_name(self):
        return self.name 
    def get_source_id(self):
        return self.source_id
    def get_file(self): 
        return self.file
    def get_path(self): 
        return self.path
    def get_dir_name(self): 
        return self.dir_name
    def is_method_component(self):
        return self.is_method
    def is_module_component(self):
        return self.is_module
    def is_file_component(self):
        return self.is_file
    def is_sub_component(self):
        return self.is_subcomponent
    
    

class FileComponent(Component):
    "a component that corresponds to a Myghty template file"

    def __init__(self, interpreter, component_source, **params):
        Component.__init__(self, interpreter, component_source, **params)

        
        self.defs = None
        self.methods = None
        self.inherit_path = None
        self.inherit_start_path = None
        self.is_file = True

        self.size = component_source.modulesize

        self.component_init()

        if self.uses_thread_local():
            self.thread_local = ThreadLocal(creator = self.thread_local_initializer)
        
        if self.uses_request_local():
            self.request_local = weakref.WeakKeyDictionary()
            
        self.subcomponents = self.defs.copy()
        self.subcomponents.update(self.methods)
        self._determine_inheritance()
        
        

    def uses_thread_local(self):raise NotImplementedError()
    def uses_request_local(self):raise NotImplementedError()

    def is_file_component(self):return True

    def get_sub_component(self, name):
        if self.subcomponents.has_key(name):
            return self.subcomponents[name]
        else:
            return None

    def _call_dynamic(self, key, m, ARGS, **params):
        """handles components with %threadlocal and/or %requestlocal sections"""
        
        context = None
        
        if self.uses_thread_local():
            if self.uses_request_local():
                if self.request_local.has_key(m):
                    context = self.request_local[m]
                else:
                    initializer = self.thread_local.get(self, m, ARGS, **params)
                    context = initializer(self, m, ARGS, **params)
                    self.request_local[m] = context
                
            else:
                context = self.thread_local.get(self, m, ARGS, **params)
            
        elif self.uses_request_local():
            if self.request_local.has_key(m):
                context = self.request_local[m]
            else:
                context = self.request_local_initializer(self, m, ARGS, **params)
                self.request_local[m] = context
                
        if context is None:
            raise exception.Error("dynamic method context not found")
        else:
            return context[key](m, ARGS, **params)


        
    def locate_inherited_method(self, method_name):
        component = self
        while component:
            if component.methods.has_key(method_name):
                return component.methods[method_name]
            component = component.parent_component

        raise exception.MethodNotFound("no such method '%s' found" % method_name)
        
    
    def _determine_inheritance(self):
    
        interpreter = self.interpreter
        source = self.component_source

        if self.flags.has_key('inherit'):
            if self.flags['inherit'] is None:
                pass
            else:
                self.inherit_path = unixpath.normpath(unixpath.join(source.dir_name, self.flags['inherit']))
        elif interpreter.use_auto_handlers:
            if source.name == interpreter.auto_handler_name:
                if source.dir_name and source.dir_name != '/':
                    self.inherit_start_path = unixpath.dirname(source.dir_name)
                else:
                    # we are the autohandler in the root directory
                    pass
            else:
                self.inherit_start_path = self.component_source.dir_name
        

    def _parent_component(self):
        """returns the parent component of this component, taking into account
        the component's inherit flag as well as the interpreter's autohandler properties"""

        try:
            request = myghty.request.instance()

            # optional - cache results of this function on the request
            #try:
            #   return request.attributes['_parent_comp_%s' % self.id]
            #except KeyError: pass
            
        except KeyError:
            # no current request for whatever reason, so call
            # load() off the interpreter
            request = None

        if self.inherit_path is not None:
            if request is not None:
                component = request.load_component(self.inherit_path, resolver_context = 'inherit')
            else:
                component = self.interpreter.load(self.inherit_path, resolver_context = 'inherit')
        elif self.inherit_start_path is not None:
            if request is not None:
                component = request.load_component(unixpath.join(self.inherit_start_path, self.interpreter.auto_handler_name), search_upwards = True, raise_error = False, resolver_context = 'inherit')
            else:
                component = self.interpreter.load(unixpath.join(self.inherit_start_path, self.interpreter.auto_handler_name), search_upwards = True, raise_error = False, resolver_context = 'inherit')
        else:
            component = None
        
        #if request is not None:
        #   request.attributes['_parent_comp_%s' % self.id] = component
            
        return component

    parent_component = property(_parent_component, lambda s, p: None)


class SubComponent(Component):
    """a component that corresponds to a <%def> or <%method> tag 
    inside a file-based component."""
    
    def __init__(self, name, owner, is_method, **params):
        Component.__init__(self, owner.interpreter, owner.component_source, **params)
        self.__name = name
        self.__owner = owner
        self.is_method = is_method
        self.is_subcomponent = True

        self.component_init()


    def _call_dynamic(self, key, m, ARGS, **params):
        return self.owner._call_dynamic(key, m, ARGS, **params)

    id = property(lambda self: self.owner.id + ":" + self.name)
    parent_component = property(lambda self: self.owner.parent_component, lambda s, p:None)
    name = property(lambda self: self.__name, lambda s,p:None)
    owner = property(lambda self: self.__owner, lambda s,p:None)
    
    def get_sub_component(self, name):return self.owner.get_sub_component(name)

    def locate_inherited_method(self, method_name):return self.owner.locate_inherited_method(method_name)
    def call_method(self, *args, **params):return self.owner.call_method(*args, **params)
    def scall_method(self, *args, **params):return self.owner.scall_method(*args, **params)




class ModuleComponent(Component):
    """A component that is a regular Python class inside of a plain Python module."""
    
    def __init__(self, interp, component_source, owner = None, do_init = True):
        Component.__init__(self, interp, component_source)
        self.__name = self.__class__.__name__
        self.is_method = True
        self.creationtime = component_source.last_modified
        
        self.methods = Registry()
        if owner is None:
            self.__owner = self
        else:
            self.__owner = owner

        if do_init:
            self.component_init()


    owner = property(lambda self: self.__owner, lambda s, p:None)
    name = property(lambda self: self.__name, lambda s, p:None)

    def _inspect_args(self, function):
        csource = self.component_source

        argspec = inspect.getargspec(function)
            
        argnames = argspec[0] or []
        defaultvalues = argspec[3] or []

        d = dict([(a, True) for a in self.interpreter.compiler().allow_globals + ['m', 'ARGS', 'self']])

        (self.required_args, self.args) = (
                [a for a in argnames[0:len(argnames) - len(defaultvalues)] if not d.has_key(a)], 
                [a for a in argnames[len(argnames) - len(defaultvalues):] if not d.has_key(a)]
                )
        
        self._allargs = argnames
        self._has_params = argspec[2] is not None

    def component_init(self):
        self.do_component_init()

        # if no new-style argument list has been set up
        if len(self.arguments) == 0:
            # and no old-style argument lists have been added
            for argtype in argtypes:
                if hasattr(self, argtype[2]):
                    break
            else:
                # then inspect the do_component_init function and figure out the args
                # that way.
                self._inspect_args(self.do_run_component.im_func)

        # tell base class to set up arguments list    
        self._convert_component_args()
        
    def do_component_init(self):pass

    def do_run_component(self, m, ARGS, **params):raise NotImplementedError()
    
    def is_module_component(self):return True

    def _find_method(self, method_name):
        if self.__class__.__name__ == method_name:
            return self

        return self.interpreter.load_module_component(self.__module__ + ":" + method_name)            
    
    def locate_inherited_method(self, method_name):
        if self.owner is self:
            return self.methods.get(method_name, createfunc = lambda: ModuleComponent._find_method(self, method_name))
        else:
            return self.owner.locate_inherited_method(method_name)
        
    
    
class HTTPModuleComponent(ModuleComponent):
    """A ModuleComponent that contains methods specific to a generic HTTP request."""

    def do_post(self, m, r, **params):
        # 'method not allowed', unless this is overridden
        m.abort(405)

    def do_get(self, m, r, **params):
        # 'method not allowed', unless this is overridden
        m.abort(405)
    
    def do_run_component(self, m, r, **params):
        methods = {
            'GET': HTTPModuleComponent.do_get,
            'POST': HTTPModuleComponent.do_post,
        }
        
        try:
            f = methods[r.method]
            return f(self, m, r, **params)
        except KeyError:
            m.abort(405)
            
class FunctionComponent(ModuleComponent):
    """a module component that "wraps" a regular Python function or method, including
    introspection of its signature to automatically produce the required_args
    and args lists """
    
    def do_component_init(self):
        csource = self.component_source

        self._inspect_args(csource.callable_)
        
        # if we are linked to a method off of an object instance,
        # see if the object instance has a do_component_init method and if the object instance
        # does not have a _component_init attribute.  call do_component_init() and 
        # set the _component_init attribute to True
        if self.component_source.has_method:
            target = self.component_source.callable_.im_self
            if hasattr(target, 'do_component_init') and not hasattr(target, '_component_init'):
                try:
                    getattr(target, 'do_component_init')(self)
                finally:
                    target._component_init = True
               
    def do_run_component(self, **params):
        if self._has_params:
            return self.component_source.callable_(**params)    
        else:
            d = {}
            for a in self._allargs:
                if params.has_key(a):
                    d[a] = params[a]
            return self.component_source.callable_(**d)
                
    def get_method_csource(self, name):    
        if not self.component_source.has_method:
            return None
        obj = self.component_source.callable_.im_self
        meth = getattr(obj, name)
        return myghty.csource.ModuleComponentSource(module=self.component_source.module, callable_=meth, name="method:" + meth.im_func.__name__)

    def get_method(self, m, name):
        cs = self.get_method_csource(name)
        return m.interpreter.load_component(cs)



import zblog.database
import zblog.util.form as form
import StringIO, os
from zblog.controller import *
import zblog.domain.actions as actions

class BootstrapController(Controller):
    """bootstrap controller, is used to receive configuration details 
    and create config.py file. this controller is only used when first 
    configuring the application."""

    @access_control(action=actions.Bootstrap())
    def __call__(self, m, ARGS, dbtype='sqlite'):
        form = self.form(ARGS, post=False)
        self.template(m, '/bootstrap/', form=form, dbtype=dbtype)
    
    @access_control(action=actions.Bootstrap())
    def bootstrap(self, m, ARGS, **kwargs):
        form = self.form(ARGS, post=True)
        if not form.is_valid():
            self.template(m, '/bootstrap/', form=form, **kwargs)
        else:            
            (logstring, error) = self.bootstrap_app(m, form) 
            self.template(m, '/bootstrap/complete.myt', log=logstring, error=error)
    
    @access_control(action=actions.Bootstrap())
    def ajax_dboptions(self, m, ARGS, dbtype):
        m.comp('/bootstrap/index.myt:dboptions', dbtype=dbtype, form=self.form(ARGS))
    
    @access_control(action=actions.Bootstrap())
    def ajax_testconnect(self, m, r, ARGS, dbtype):
        dbform = self.form(ARGS)['dbform']
        dbform.set_request(ARGS)
        kwargs = {}
        for e in dbform:
            kwargs[e.name] = e.value
        try:            
            connect = zblog.database.test_connection(dbtype, kwargs)
            error = None
        except Exception, e:
            error = e
        m.comp('/bootstrap/index.myt:dboptions', dbtype=dbtype, form=self.form(ARGS), connected=(error is None), error=error)
    
    def form(self, ARGS, post=False):
        db_descriptors = zblog.database.dbtypes()
        opt = [(None, '(select)')] + [(desc['name'], desc['description']) for desc in db_descriptors]
        f = form.Form('bootstrap',[
            form.FormField('adminuser', required=True, default='Administrator'),
            form.FormField('adminpw', required=True),
            form.FormField('dbtype', required=True, default='sqlite', data=opt)
        ])

        dbform = form.SubForm('dbform', [])
        f.append(dbform)
        
        dbtype = ARGS.get('dbtype', 'sqlite')
        desc = zblog.database.get_descriptor(dbtype)
        if desc is not None:
            dbform.description = desc['description']
            f['dbtype'].value=dbtype
            for field in desc['arguments']:
                if dbtype == 'sqlite' and field[0] == 'filename':
                    default = './data/zblog.db'
                else:
                    default = field[2]
                dbform.append(form.FormField(field[0], description=field[1], default=default, required=True))

        f.set_request(ARGS, validate=post)
        return f


    def bootstrap_app(self, m, form):
        """when all the arguments are assembled, this method 
        writes out a config file and calls the appropriate 
        modules to create the database schema."""
        s = StringIO.StringIO()
        dbtype = form['dbtype'].value
        desc = zblog.database.get_descriptor(dbtype)
        s.write("""
database = dict(
    driver='%s',
    echo=True,
""" % dbtype)
        subform = form['dbform']
        for arg in desc['arguments']:
            s.write("    %s='%s',\n" % (arg[0], subform[arg[0]].value))
        s.write(""")""")
        
        logger =StringIO.StringIO()
        # write configuration file
        logger.write("Creating configuration file '%s'\n" % m.interpreter.attributes['config_file'])
        f = file(m.interpreter.attributes['config_file'], 'w')
        f.write(s.getvalue())
        f.close()

        try:
            # initialize application config
            zblog.load_config(m.interpreter.attributes['config_file'])

            # create database
            logger.write("Creating application tables\n")
            zblog.database.init_database(admin_username=form['adminuser'].value, admin_password=form['adminpw'].value, logger=logger)
            error = False
        except Exception, e:
            # boom - delete config file
            logger.write("Error occurred: '%s'\ndeleting config file\n" % str(e))
            zblog.reset_config()
            os.remove(m.interpreter.attributes['config_file'])
            error = True

        return (logger.getvalue(), error)

        
index = BootstrapController()



# $Id: objgen.py 2133 2006-09-06 18:52:56Z dairiki $
# objgen.py - generates object files for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

import re, string, types, time
from myghty.util import *
import myghty.exception as exception

class ObjectGenerator:
    """ generates myghty object files.
    
    uses the visitor pattern combined with the Compiler's flyweight pattern to 
    efficiently loop through a tokenized page object and produce appropriate code.
    
    an ObjectGenerator is stateful and is not threadsafe.  the clone() method can be used
    to create copies of this object for use in multiple threads.
    """
    
    def visit_code_block(self, block_obj, context):raise NotImplementedError()
    def visit_code_single_line(self, block_obj, context):raise NotImplementedError()
    def visit_text_line(self, block_obj, context):raise NotImplementedError()
    def visit_component_block(self, compiled):raise NotImplementedError()
    def visit_def_block(self, compiled):raise NotImplementedError()
    def visit_method_block(self, compiled):raise NotImplementedError()
    def visit_closure_block(self, compiled):raise NotImplementedError()
    def visit_substitution(self, block_obj, context):raise NotImplementedError()
    def visit_component_call(self, block_obj, context):raise NotImplementedError()
    def visit_component_content_call(self, block_obj, context):raise NotImplementedError()
    def visit_component_content_call_end(self, block_obj, context):raise NotImplementedError()
    
    def generate(self, compiler, compiled, stream):raise NotImplementedError()
    
    def clone(self, **params):
        """creates a clone of this ObjectGenerator.  allow the Prototype pattern
        to be used in creating generators for other threads."""
        raise NotImplementedError()


class PythonGenerator:


    def clone(self, **params):
        """creates a clone of this ObjectGenerator.  allow the Prototype pattern
        to be used in creating generators for use in other threads."""
        return PythonGenerator()

    def generate(self, compiler, compiled, stream):
        self.printer = PythonPrinter(stream)
        self.compiler = compiler
        self.compiled = compiled
        self.textbuffer = ''
        self.indentstack = []
        self.in_block = False
        self.last_asserted_line = -1
        
        self.uses_shared = (compiled.block_has_code("shared"))
        self.uses_threadonce = (compiled.block_has_code("threadonce"))
        
        # call the initial accept_visitor that will start the 
        # traversal of the whole compiled object
        compiled.accept_visitor(self)
        self.printer.close()

    def visit_code_block(self, block_obj, context):
        """handles a <%python> block, adjusting indent to match the
        current indent level."""
        
        self._flush_text()
        lines = block_obj.get_lines(context)

        self.printer.print_comment("BEGIN CODE BLOCK")
        self._assert_line_comment(block_obj, context)
        linenum = block_obj.get_line_number(context)
        
        for line in lines:
            line = self.compiler.post_process_python(line)
            #self.printer.add_normalized_block_line(self._line_comment(linenum))
            #linenum += 1
            self.printer.add_normalized_block_line(line)

        self.printer.print_comment("END CODE BLOCK")
    
    def visit_text_line(self, block_obj, context):
        """handles regular lines of text.
        
         Text line printing is buffered so that consecutive lines of text
         can be combined into a single print statement"""
         
        if not self.textbuffer: self._assert_line_comment(block_obj, context)
        self.textbuffer += self.compiler.post_process_text(block_obj.get_line(context))

    def visit_component_call(self, block_obj, context):
        self._generate_component_call(block_obj, context, content = False)
        
    def visit_component_content_call(self, block_obj, context):
        self._print_python("def %s():" % "component_content", block_obj, context)
        self._assert_line_comment(block_obj, context)
        self._cc_line = block_obj.get_line_number(context)
        
    def visit_component_content_call_end(self, block_obj, context):
        self._generate_component_call(block_obj, context, content = True)

    def _generate_component_call(self, block_obj, context, content = False):
        name = block_obj.get_component(context)

        self._assert_line_comment(block_obj, context)

        if self.closures.has_key(name):
            funcname = 'm.closure_with_content'
            args = [name]
        else:
            funcname = 'm.execute_component'
            args = ["'%s'" % name]
        if block_obj.get_args(context):
            args.append("args = argdict(%s)" % block_obj.get_args(context))
        if content is True:
            args.append("content = component_content")
            if block_obj.get_line_number(context) == self._cc_line:
                self._print_python("pass", block_obj, context)
            self._print_python(["", "%s(%s)" % (funcname, string.join(args, ", "))],
                        block_obj, context)
        else:
            self._print_python("%s(%s)" % (funcname, string.join(args, ", ")),
                        block_obj, context)
        
    def visit_code_single_line(self, block_obj, context):
        self._assert_line_comment(block_obj, context)
        self._print_python(block_obj.get_line(context), block_obj, context, normalize_indent = True)


    def visit_substitution(self, block_obj, context):
        escapes = block_obj.get_escapes(context)
        if escapes:
            code = "m.write(m.apply_escapes(%s, %s))" % (block_obj.get_line(context), repr(escapes))
        else:
            code = "m.write(%s)" % block_obj.get_line(context)
        
        self._assert_line_comment(block_obj, context)
        self._print_python(code, block_obj, context)
    
    
    def visit_component_block(self, compiled):
        self._generate_module_header(compiled)
        
        for block in compiled.get_named_blocks():
            block.accept_visitor(self)

        self._generate_component_class(compiled)
        

    def visit_def_block(self, compiled):
        self._generate_component_class(compiled, is_subcomponent = True, is_method = False)
            
    def visit_method_block(self, compiled):
        self._generate_component_class(compiled, is_subcomponent = True, is_method = True)


    def visit_closure_block(self, block_obj, context):
        compiled = block_obj.get_compiled(context)

        indent = self.printer.indentstring * self.printer.indent + " " * len(compiled.name)

        self.closures[compiled.name] = True
        
        def format_arg(arg):
            if not arg.required:
                return "\n%s%s = %s, " % (indent, arg.name, arg.default)
            else:
                return "\n%s%s, " % (indent, arg.name)

        args = []
        
        reqarg = [arg for arg in compiled.args.values() if arg.required]
        if len(reqarg):
            args.append(string.join(map(format_arg, reqarg)))

        nonreqarg = [arg for arg in compiled.args.values() if not arg.required]
        if len(nonreqarg):
            args.append(string.join(map(format_arg, nonreqarg)))

        args += ["\n" + indent + "**_comp_params"]

        code = "def %s(%s):" % (compiled.name, string.join(args))
        self._print_python(code, block_obj, context)
        self._generate_run_body(compiled)


    # generation methods.  each spits out various parts of the module 
    # file.  they are broken up to handle some freaky rearrangements that occur
    # based on if the component uses <%shared>, <%filter>, <%cleanup> etc.
    # <%shared> is the craziest one by far.
    
    def _generate_module_header(self, compiled):
        """generates the top header of the file, the <%once> section, and the dynamic subs if
        the module has a <%shared> block"""
        
        self.printer.print_comment("File: %s CompilerID: %s Timestamp: %s" % (compiled.fileid, self.compiler.get_object_id(), time.asctime()))

        if compiled.encoding is not None:
            self.printer.print_comment("-*- encoding: %s -*-"
                                       % compiled.encoding)
            

        self.printer.print_python_line("from myghty.component import Component, FileComponent, SubComponent")
        
        self.printer.print_python_line("import myghty.request as request")
        self.printer.print_python_line("import myghty.args")
        self.printer.print_python_line("from myghty.util import *")

        self.printer.print_python_line("_CREATION_TIME = %d" % time.time())
        
        self.printer.print_python_line("_MAGIC_NUMBER = %s"
                                       % repr(self.compiler.get_magic_number()))
        if compiled.block_has_code("once"):
            self._output_block(compiled, "once")

        if self.uses_threadonce:
            self._push_indent()
            self._generate_python_def(compiled, "thread_local_initializer", append_globals = True, append_component = False, args = ['self'])
            self._output_block(compiled, "threadonce")
            
            if self.uses_shared:
                self._push_indent()
                self._generate_python_def(compiled, "request_local_initializer", append_globals = True, append_component = False, args = ['self'])
                self._output_block(compiled, "shared")
                
            self._generate_dynamic_subs(compiled)
            
            if self.uses_shared:
                self._pop_indent()
                self.printer.print_raw_line("\n")
                self.printer.print_python_line("return request_local_initializer")
            
            self._pop_indent()
            
        elif self.uses_shared:
            self._push_indent()
            self._generate_python_def(compiled, "request_local_initializer", append_globals = True, append_component = False, args = ['self'])
            self._output_block(compiled, "shared")

            self._generate_dynamic_subs(compiled)
            
            self._pop_indent()  
    
    def _generate_dynamic_subs(self, compiled): 
        for block in compiled.get_named_blocks() + [compiled]:
            self._push_indent()
            self._generate_run_define(block, methodname = ("run_%s" % self._format_class_name(block.name)), is_dynamic_method = True)
            self._generate_run_body(block)
            self._pop_indent()
        self.printer.print_python_line("return {")
        for block in compiled.get_named_blocks() + [compiled]:
            self.printer.print_python_line("   '%s' : %s," % (self._format_class_name(block.name),("run_%s" % self._format_class_name(block.name)) ))
        self.printer.print_python_line("}")
    
    def _generate_component_class(self, compiled, **params):
        """generates the class definition of a component, all methods within the class,
        and a static accessor function if the component is the lead component"""
        
        self._push_indent()
        self._generate_init(compiled, **params)

        dynamic = (self.uses_shared or self.uses_threadonce)
        self._generate_run_define(compiled, add_dynamic_call = dynamic)
        if not dynamic:
            self._generate_run_body(compiled)

        self._pop_indent()
    
        if compiled.in_main:
            self.printer.print_raw_line("\n")
            self.printer.print_python_line("def get_component(interpreter, csource): return %s(interpreter, csource)" % self._format_class_name(compiled.name))

    def _generate_subcomponent_constructor(self, compiled, is_method):
        """generates a constructor call to create a subcomponent (def or method)"""
        
        return "'%s' : %s('%s', self, is_method = %s)" % (
            compiled.name,
            self._format_class_name(compiled.name),
            compiled.name,
            repr(is_method)
            )
        
    def _generate_init(self, compiled, is_subcomponent = False, is_method = False):
        """generates the __init__ method for a component """
        
        if is_subcomponent:
            superclass = "SubComponent"
        else:
            superclass = "FileComponent"

        self.printer.print_raw_line("\n")
        self.printer.print_python_line("class %s(%s):" % (self._format_class_name(compiled.name), superclass))

        self.printer.print_python_line("def do_component_init(self):")

        self.printer.print_python_line("self.defs = {%s}" % 
                string.join(map(lambda c: self._generate_subcomponent_constructor( c, False), 
                    compiled.get_named_blocks("def")), ",\n"))

        self.printer.print_python_line("self.methods = {%s}" % 
                string.join(map(lambda c: self._generate_subcomponent_constructor( c, True), 
                compiled.get_named_blocks("method")), ",\n"))

        self.printer.print_python_line("self.flags = %s" % self._repr_key_value(compiled.flags))
        self.printer.print_python_line("self.attr = %s" % self._repr_key_value(compiled.attr))
        self.printer.print_python_line("self.arguments = [%s]" % string.join(["myghty.args." + repr(arg) for arg in compiled.args.values()], ",\n"))
        self.printer.print_python_line("self.creationtime = _CREATION_TIME")
        
        if compiled.in_main:
            if self.uses_threadonce:
                    self.printer.print_python_line("self.thread_local_initializer = thread_local_initializer")
            elif self.uses_shared:
                self.printer.print_python_line("self.request_local_initializer = request_local_initializer")

        if compiled.block_has_code("filter"):
            self._generate_python_def(compiled, "filter", append_globals = True, append_component = False, args = ['f'])
            self._output_block(compiled, "filter")
            self.printer.print_python_line("")
            self.printer.print_python_line("self.filter = self._init_filter_func(filter)")
        elif compiled.flags.has_key('trim'):
            self.printer.print_python_line("self.filter = self._init_filter_func()")
            
        self.printer.print_python_line("")
        
        self.printer.print_python_line("def uses_request_local(self):return %s" % repr(self.uses_shared and compiled.in_main))
        self.printer.print_python_line("def uses_thread_local(self):return %s" % repr(self.uses_threadonce and compiled.in_main))
        

    def _repr_key_value(self, keyvalue):
        return ("{" +
            string.join(map(lambda k: "'%s' : %s" % (k, keyvalue[k]), keyvalue.keys()), ",\n")
        + "}")
            
    def _generate_run_define(self, compiled, methodname = "do_run_component", is_dynamic_method = False, add_dynamic_call = False):

        self.closures = {}

        (args, indent) = self._generate_method_args(compiled, methodname, append_globals = (is_dynamic_method or not add_dynamic_call), append_component = (is_dynamic_method or not add_dynamic_call))
        if not is_dynamic_method:
            defargs = ['self, '] + args
        else:
            defargs = args
        
        self.printer.print_comment("BEGIN BLOCK args")
        if compiled.start_blocks['args'] != -1: self.printer.print_comment(self._line_comment(compiled.start_blocks['args']))
        self.printer.print_python_line("def %s(%s\n%s):" % (methodname, string.join(defargs), indent))
        self.printer.print_comment("END BLOCK args")
    
        if add_dynamic_call:
            self.printer.print_python_line("return self._call_dynamic('%s', %s)" % 
                (
                    self._format_class_name(compiled.name),
                    string.join(args)
                )
            )

    def _generate_python_def(self, compiled, methodname, append_globals, append_component, args = None):
        (genargs, indent) = self._generate_method_args(compiled, methodname, append_globals = append_globals, append_component = append_component)
        if args is not None:
            genargs = [arg + ", " for arg in args] + genargs
        self.printer.print_python_line("def %s(%s\n%s):" % (methodname, string.join(genargs), indent))
        
    def _generate_method_args(self, compiled, methodname, append_globals, append_component):
        
        indent = self.printer.indentstring * self.printer.indent + " " * len(methodname)

        # we're trying to print the args with linebreaks, for an <%ARGS> section that was defined like
        # "g ='foo' #define g to be foo" , but this doesnt really work anyway since you need to get the commas 
        # in between where the comment starts....
        def format_arg(arg):
            if not arg.required:
                return "\n%s%s = %s, " % (indent, arg.name, arg.default)
            else:
                return "\n%s%s, " % (indent, arg.name)

        args = ['m, ', 'ARGS, ']

        if append_globals:
            # append global arguments to arg list.  the most ordinary is
            # "r" for the HTTP request.  "s" for session would be nice as well.
            args += map(lambda x: "%s, " % x, compiled.compiler.allow_globals)

        if append_component:
            reqarg = [arg for arg in compiled.args.values() if arg.required]
            if len(reqarg):
                args.append(string.join(map(format_arg, reqarg)))

            nonreqarg = [arg for arg in compiled.args.values() if not arg.required]
            if len(nonreqarg):
                args.append(string.join(map(format_arg, nonreqarg)))

            args += ["\n" + indent + "**_comp_params"]
        else:
            args += ['**_comp_params']

        return (args, indent)

        
    def _generate_run_body(self, compiled):
        
        has_blocks = False
        
        # generate blocks in order
        if compiled.block_has_code("cleanup"):
            # increments indent one more level
            self.printer.print_python_line("try:")
        
        for key in ('init', 'body', 'cleanup'):
            if not compiled.block_has_code(key):
                continue
                
            if key == 'cleanup':
                self.printer.print_python_line("finally:")
                self.printer.print_python_line("pass")
                
            self._output_block(compiled, key)
            has_blocks = True

        if not has_blocks:
            self.printer.print_python_line("pass")
            
        self.printer.print_python_line("")
        
        if compiled.block_has_code("cleanup"):
            # decrement indent one more level
            self.printer.print_python_line("")
        

    def _output_block(self, compiled, block_type):
        """traverses the lines of flyweight objects in a block 
        object (such as init, once, body) and outputs python code"""
        
        self._push_indent()
        iter = compiled.get_block_iterator(block_type)
        has_code = False
        
        outermost_block = False
        if not self.in_block:
            self.printer.print_comment("BEGIN BLOCK %s" % block_type)
            self.in_block = True
            outermost_block = True
        
        for code in iter:
            has_code = True
            code.accept_visitor(self, iter)

        if has_code:
            self._flush_text()

        if outermost_block:
            self.printer.print_comment("END BLOCK %s" % block_type)
            self.in_block = False
            
        self._pop_indent()
        self.printer.clear_whitespace_stack()
        return has_code


    def _print_python(self, code, block_obj, context, normalize_indent = False):
        """prints python code, performing the convenience steps of flushing
        the existing text buffer, printing line number tracking comments,
        and post processing of python code"""
        
        self._flush_text()
        self._assert_line_comment(block_obj, context)

        if type(code) == types.ListType:
            for c in code:
                self.compiler.post_process_python(c)
                self.printer.print_python_line(c, normalize_indent = normalize_indent)
        else:
            self.compiler.post_process_python(code)
            self.printer.print_python_line(code, normalize_indent = normalize_indent)
        
    def _assert_line_comment(self, block_obj, context):
        if block_obj.get_line_number(context) != self.last_asserted_line:
            self.printer.print_comment(self._line_comment(block_obj.get_line_number(context)))
            self.last_asserted_line = block_obj.get_line_number(context)

    def _line_comment(self, linenumber):
        return "SOURCE LINE %s" % (linenumber)

    def _flush_text(self):
        if not self.textbuffer: return
        buffer = self.compiler.post_process_text(self.textbuffer)
        # escape single quotes and backslashes
        buffer = re.sub(r"(['\\])", r"\\\1", buffer)
        if self.compiler.disable_unicode:
            self.printer.print_python_line("m.write('''%s''')" % buffer)
        else:
            self.printer.print_python_line("m.write(u'''%s''')" % buffer)
        self.textbuffer = ''

    def _format_class_name(self, name):
        name = re.sub(r"\.", "_", name)
        name = re.sub(r"[^\d\w_]", "", name)
        return "_" + name

    def _push_indent(self):
        """stores the current indentation level of the Python printer on a stack."""
        self.indentstack.append(self.printer.indent)
        
    def _pop_indent(self):
        """restores the indentation level of the Python printer to what we last
        stored."""
        indent = self.indentstack.pop()
        self.printer.indent = indent

        
class PythonPrinter:
    """prints Python code, keeping track of indentation level.  PythonPrinter has two
    basic modes, "print_python_line" mode which prints one line at a time and calculates
    whitespace on the fly, and "add_normalized_block_line" mode where you add lines to a 
    buffer, and then at the end print them all out as they were stored, 
    except relative to the current indent."""
    
    def __init__(self, stream):
        # the indentation counter
        self.indent = 0
        
        # a stack storing information about why we incremented 
        # the indentation counter, to help us determine if we
        # should decrement it
        self.indent_detail = []
        
        # the string of whitespace multiplied by the indent
        # counter to produce a line
        self.indentstring = "       "
        
        # a stack of whitespace we pulled from "normalized" 
        # Python lines to track when the indentation counter should
        # be incremented or decremented
        self.spacestack = []
        
        # the stream we are writing to
        self.stream = stream
        
        # a list of lines that represents a buffered "block" of code,
        # which can be later printed relative to an indent level 
        self.line_buffer = []
        
        # boolean indicating if we are in "print_python_line" mode or
        # "add_normalized_block_line" mode
        self.in_indent_lines = False
        
        self._reset_multi_line_flags()
        
    def print_python_line(self, line, normalize_indent = False, is_comment = False):
        """prints a line to the output buffer, preceded by a blank indentation
        string of proportional size to the current indent counter.  
        
        If the line ends with a colon, the indentation counter is incremented after
        printing.  If the line is blank, the indentation counter is decremented.
        
        if normalize_indent is set to true, the line is printed
        with its existing whitespace "normalized" to the current indentation 
        counter; additionally, its existing whitespace is measured and
        compared against a stack of whitespace strings grabbed from other
        normalize_indent calls, which is used to adjust the current indentation 
        counter.
        
        basically, "normalize_indent" is used with % lines in a template, 
        and non-normalize_indent is used by the objectgenerator's internally 
        generated lines of code.
        """

        if not self.in_indent_lines:
            self.flush_normalized_block()
            self.in_indent_lines = True

        decreased_indent = False
    
        if (
            re.match(r"^\s*#",line) or
            re.match(r"^\s*$", line)
            ):
            hastext = False
        else:
            hastext = True

        if normalize_indent:
            # determine the actual amount of whitespace preceding the
            # line of code.  check it against a stack of whitespace
            # and push it on if it is of greater indent and also
            # has some non-whitespace text, or pop 
            # the existing whitespace if it is of lesser indent
        
            line = string.expandtabs(line)
            space = re.match(r"^([ ]*)", line).group(1)

            # see if we have any whitespace already 
            if len(self.spacestack) == 0:
                if hastext:
                    self.spacestack.append(space)
            else:
                if len(space) > len(self.spacestack[-1]):
                    if hastext:
                        self.spacestack.append(space)

                elif len(space) < len(self.spacestack[-1]):
                    self.spacestack.pop()
                    if self.indent > 0: 
                        self.indent -=1
                        if len(self.indent_detail) == 0:  raise exception.Compiler("Too many whitespace closures")
                        self.indent_detail.pop()
                        decreased_indent = True

            line = string.lstrip(line)
        else:
            # not normalizing indentation.  no whitespace should be present.
            space = None
        
        # see if this line should decrease the indentation level
        if (not decreased_indent and 
            not is_comment and 
            (not hastext or self._is_unindentor(line, space))
            ):
            
            if self.indent > 0: 
                self.indent -=1
                # if the indent_detail stack is empty, the user
                # probably put extra closures - the resulting
                # module wont compile.  
                if len(self.indent_detail) == 0:  raise exception.Compiler("Too many whitespace closures")
                self.indent_detail.pop()
            
        # write the line    
        self.stream.write(self._indent_line(line) + "\n")
        
        # see if this line should increase the indentation level.
        # note that a line can both decrase (before printing) and 
        # then increase (after printing) the indentation level.

        if re.search(r":[ \t]*(?:#.*)?$", line):
            # increment indentation count, and also
            # keep track of what the keyword was that indented us,
            # if it is a python compound statement keyword
            # where we might have to look for an "unindent" keyword
            match = re.match(r"^\s*(if|try|elif|while|for)", line)
            if match:
                # its a "compound" keyword, so we will check for "unindentors"
                indentor = match.group(1)
                self.indent +=1
                self.indent_detail.append((space, indentor))
            else:
                indentor = None
                # its not a "compound" keyword.  but lets also
                # test for valid Python keywords that might be indenting us,
                # else assume its a non-indenting line
                m2 = re.match(r"^\s*(def|class|else|elif|except|finally)", line)
                if m2:
                    self.indent += 1
                    self.indent_detail.append((space, indentor))
        
    
    def _is_unindentor(self, line, space):
        """looks at the keyword and its whitespace that was most recently 
        responsible for incrementing the indent,
        and compares against the keyword (if any) and whitespace for the given line,
        to see if an unindent operation should occur"""
                
        # no indentation detail has been pushed on; return False
        if len(self.indent_detail) == 0: return False

        (indspace, indentor) = self.indent_detail[-1]
        
        # the last indent keyword we grabbed is not a 
        # compound statement keyword; return False
        if indentor is None: return False
        
        # the indentation from the last indent keyword we
        # grabbed does not match this current keword; return False
        if indspace != space: return False
        
        # if the current line doesnt have one of the "unindentor" keywords,
        # return False
        match = re.match(r"^\s*(else|elif|except|finally)", line)
        if not match: return False
        
        # whitespace matches up, we have a compound indentor,
        # and this line has an unindentor, this
        # is probably good enough
        return True
        
        # should we decide that its not good enough, heres
        # more stuff to check.
        #keyword = match.group(1)
        
        # match the original indent keyword 
        #for crit in [
        #   (r'if|elif', r'else|elif'),
        #   (r'try', r'except|finally|else'),
        #   (r'while|for', r'else'),
        #]:
        #   if re.match(crit[0], indentor) and re.match(crit[1], keyword): return True
        
        #return False
        
    def print_comment(self, comment):
        self.print_python_line("# " + comment, is_comment = True)
        
    def _indent_line(self, line, stripspace = ''):
        return re.sub(r"^%s" % stripspace, self.indentstring * self.indent, line)

    def _reset_multi_line_flags(self):
        (self.backslashed, 
        self.triplequoted) = (False, False) 
        
    def _in_multi_line(self, line):
        # we are only looking for explicitly joined lines here,
        # not implicit ones (i.e. brackets, braces etc.).  this is just
        # to guard against the possibility of modifying the space inside 
        # of a literal multiline string with unfortunately placed whitespace
         
        current_state = (self.backslashed or self.triplequoted) 
                        
        if re.search(r"\\$", line):
            self.backslashed = True
        else:
            self.backslashed = False
            
        triples = len(re.findall(r"\"\"\"|\'\'\'", line))
        if triples == 1 or triples % 2 != 0:
            self.triplequoted = not self.triplequoted

            
        return current_state


    def print_raw_line(self, line, indentlevel = 0):
        """adds a line to the deffered print buffer with the "multiline" flag set to true,
        so that this line will come out with its original indentation intact."""
        
        self.in_indent_lines = False
        self.line_buffer.append([True, line])


    def add_normalized_block_line(self, line):
        """adds a line to the deferred print buffer.
        
        When the deferred print buffer is flushed, its lines will be scanned for the
        initial whitespace amount.  lines with that much or greater whitespace
        will have their whitespace adjusted to line up with the whitespace that has
        been established by the last print_indent_line call.  Lines within continued 
        triplequotes and backslashes will remain unaffected.....if all goes well.
        
        This method is basically used for <%python> blocks.
        """
                
        self.in_indent_lines = False
        
        # append a record consisting of a multiline flag
        # as well as the line itself
        self.line_buffer.append([False, line])
        
    
    def flush_normalized_block(self):
        stripspace = None
        self._reset_multi_line_flags()
        
        for entry in self.line_buffer:
            if self._in_multi_line(entry[1]):
                self.stream.write(entry[1] + "\n")
            else:
                entry[1] = string.expandtabs(entry[1])
                if stripspace is None and re.search(r"[^ \t]", entry[1]):
                    stripspace = re.match(r"^([ \t]*)", entry[1]).group(1)
    
                self.stream.write(self._indent_line(entry[1], stripspace) + "\n")
            

            
        self.line_buffer = []
        self._reset_multi_line_flags()

    def clear_whitespace_stack(self):
        self.spacestack = []

    def close(self):
        self.flush_normalized_block()



# test/test_encoding.py
"""Test that stuff in myghty.requestbuffer
"""

import unittest, os, sys, codecs, StringIO, cStringIO, cPickle, warnings
import testbase
from myghty import importer, util
from myghty.requestbuffer import \
     UnicodeRequestBuffer, StrRequestBuffer, MismatchedPop

def _is_ascii_superset(encoding):
    ASCII_CHARS = ''.join(map(chr, range(128)))
    try:
        return ASCII_CHARS.decode(encoding) == ASCII_CHARS.decode('ascii')
    except UnicodeDecodeError:
        return False

if not _is_ascii_superset(sys.getdefaultencoding()):
    warnings.warn("System default encoding is not a subset of ASCII. "
                  "Many of these tests will fail.",
                  UserWarning)

REPLACEMENT_CHARACTER = u'\ufffd'

class TestBuffer:
    """A StringIO which supports only a limited set of file operations.
    """
    def __init__(self):
        self.buf = StringIO.StringIO()

    def write(self, text):
        self.buf.write(text)

    def __str__(self):
        return self.buf.getvalue()

def test_filter(text):
    return "f(%s)" % text

class _RequestBufferTests(object):
    
    def setUp(self):
        self.outputBuffer = TestBuffer()

    output = property(lambda self: str(self.outputBuffer))

    def testBuffering(self):
        buf = self.getRequestBuffer()
        buf.write("a")
        self.failUnlessEqual(self.output, "a")
        buf.push_buffer()
        buf.write("b")
        self.failUnlessEqual(self.output, "a")
        buf.push_buffer()
        buf.write("c")
        self.failUnlessEqual(self.output, "a")
        buf.pop_buffer()
        self.failUnlessEqual(self.output, "a")
        buf.pop_buffer()
        self.failUnlessEqual(self.output, "abc")

    def testBufferNoFlush(self):
        buf = self.getRequestBuffer()
        buf.write("a")
        self.failUnlessEqual(self.output, "a")
        buf.push_buffer()
        buf.write("b")
        buf.pop_buffer(discard=True)
        self.failUnlessEqual(self.output, "a")

    def testFilter(self):
        buf = self.getRequestBuffer()
        buf.push_filter(test_filter)
        buf.write("a")
        buf.write("b")
        self.failUnlessEqual(self.output, "f(a)f(b)")
        buf.push_buffer()
        buf.write("c")
        buf.write("d")
        buf.pop_buffer()
        self.failUnlessEqual(self.output, "f(a)f(b)f(cd)")
        buf.pop_filter()
        buf.write("e")
        self.failUnlessEqual(self.output, "f(a)f(b)f(cd)e")
        
    def testCaptureBuffer(self):
        buf = self.getRequestBuffer()
        buf.push_buffer()
        buf.write('a')
        self.failUnlessEqual(self.output, '')

        cbuf = util.StringIO()
        buf.push_capture_buffer(cbuf)
        buf.write('b')
        self.failUnlessEqual(cbuf.getvalue(), "b")
        buf.flush()
        self.failUnlessEqual(self.output, '')
        
        buf.push_capture_buffer()
        buf.write("c")
        cbuf2 = buf.pop_capture_buffer()
        self.failUnlessEqual(cbuf2.getvalue(), "c")

        self.failUnlessEqual(cbuf, buf.pop_capture_buffer())
        self.failUnlessEqual(cbuf.getvalue(), "b")
        self.failUnlessEqual(self.output, '')
        buf.pop_buffer()
        self.failUnlessEqual(self.output, 'a')

    def testFlush(self):
        buf = self.getRequestBuffer()
        buf.push_buffer()
        buf.write("howdy")
        self.failUnlessEqual(self.output, "")
        buf.flush()
        self.failUnlessEqual(self.output, "howdy")

    def testMismatchedPop(self):
        buf = self.getRequestBuffer()
        self.failUnlessRaises(MismatchedPop, buf.pop_buffer)
        buf.push_buffer()
        self.failUnlessRaises(MismatchedPop, buf.pop_filter)
        buf.pop_buffer()

    def testSaveAndRestoreState(self):
        buf = self.getRequestBuffer()
        buf.push_filter(test_filter)
        state = buf.get_state()
        buf.push_buffer()
        buf.write('b')
        self.failUnlessEqual(self.output, '')
        buf.pop_to_state(state)
        self.failUnlessEqual(self.output, 'f(b)')
        buf.write('a')
        self.failUnlessEqual(self.output, 'f(b)f(a)')
        
class StrRequestBufferTests(_RequestBufferTests, unittest.TestCase):
    
    def getRequestBuffer(self):
        return StrRequestBuffer(self.outputBuffer)

    def testWriteUnicodeFails(self):
        buf = self.getRequestBuffer()
        self.failUnlessRaises(UnicodeEncodeError,
                              buf.write, REPLACEMENT_CHARACTER)

    def testBinarySafe(self):
        buf = self.getRequestBuffer()
        all_chars = ''.join(map(chr, range(256)))
        buf.write(all_chars)
        self.failUnlessEqual(self.output, all_chars)
        
    def testWriteUnicodeOkay(self):
        buf = self.getRequestBuffer()
        buf.write(u'abc')
        self.failUnlessEqual(self.output, 'abc')
        
    def testFilterUnicodeOutput(self):
        def filt(text):
            if text == 'a':
                return REPLACEMENT_CHARACTER
            else:
                return unicode(text) * 2
        buf = self.getRequestBuffer()
        buf.push_filter(filt)
        self.failUnlessRaises(UnicodeEncodeError, buf.write, "a")
        buf.write('b')
        self.failUnlessEqual(self.output, "bb")
        
    def testChangeOutputEncodingFails(self):
        buf = self.getRequestBuffer()
        self.failUnless(buf.encoding is None)
        self.failUnlessRaises(AttributeError, setattr, buf, 'encoding', 'ascii')
    def testChangeEncodingErrorsFails(self):
        buf = self.getRequestBuffer()
        self.failUnless(buf.errors is None)
        self.failUnlessRaises(AttributeError, setattr, buf, 'errors', 'strict')


    
class UnicodeRequestBufferTests(_RequestBufferTests, unittest.TestCase):

    def setUp(self):
        self.outputBuffer = TestBuffer()

    output = property(lambda self: str(self.outputBuffer))

    def getRequestBuffer(self, encoding='ascii', errors='strict'):
        return UnicodeRequestBuffer(self.outputBuffer, encoding, errors)
    
    def testBufferMixedEncoding(self):
        buf = self.getRequestBuffer('latin1')
        buf.push_buffer()
        buf.write("a")
        buf.write(u'b')
        buf.write(unicode("\xc2\xa0", 'utf8'))
        self.failUnlessEqual(self.output, '')
        buf.pop_buffer()
        self.failUnlessEqual(self.output, 'ab\xa0')

    def testBufferMixedEncodingNoFlush(self):
        buf = self.getRequestBuffer('latin1')
        buf.push_buffer()
        buf.write("a")
        buf.write(u'b')
        buf.write(unicode("\xc2\xa0", 'utf_8'))
        self.failUnlessEqual(self.output, '')
        buf.pop_buffer(discard=True)
        self.failUnlessEqual(self.output, '')

    def testFilterUnicodeOutput(self):
        def filt(text):
            return u'\u20ac'

        buf = self.getRequestBuffer('iso-8859-15')
        buf.push_filter(filt)
        buf.write("a")
        self.failUnlessEqual(self.output, "\xa4")
        
    def testRecoding(self):
        buf = self.getRequestBuffer('utf_8')
        buf.write(unicode('\xa0', 'latin1'))
        self.failUnlessEqual(self.output, '\xc2\xa0')

    def testRecodingError(self):
        buf = self.getRequestBuffer('utf_8')
        buf.push_capture_buffer(buffer=cStringIO.StringIO())
        # Can't recode &nbsp; to ascii
        self.failUnlessRaises(UnicodeError,
                              buf.write, unicode('\xa0', 'latin1'))
        buf.write('a')
        cbuf = buf.pop_capture_buffer()
        self.failUnlessEqual(cbuf.getvalue(), 'a')
        self.failUnlessEqual(self.output, '')

    def testBufferIsEncodingTransparent(self):
        buf = self.getRequestBuffer('utf_8')
        buf.push_buffer()               # This buffer does not encode to ascii
        buf.write(unicode("\xa0", 'latin1'))
        self.failUnlessEqual(self.output, '')
        buf.flush()
        self.failUnlessEqual(self.output, '\xc2\xa0')

    def testUnicodeWrites(self):
        buf = self.getRequestBuffer('iso-8859-15')
        EURO_SIGN = u'\u20ac'
        buf.write(EURO_SIGN)
        self.failUnlessEqual(self.output, '\xa4')

        # There's no euro symbol in latin1
        buf = self.getRequestBuffer('iso-8859-1')
        self.failUnlessRaises(UnicodeEncodeError,
                              buf.write, EURO_SIGN)

    def testChangeOutputEncoding(self):
        nbsp = unicode('\xa0', 'latin1')
        
        buf = self.getRequestBuffer('latin1')
        self.failUnlessEqual(buf.encoding, 'latin1')
        buf.write(nbsp)
        self.failUnlessEqual(self.output, '\xa0')

        buf.set_encoding('utf-8')
        self.failUnlessEqual(buf.encoding, 'utf-8')
        buf.write(nbsp)
        self.failUnlessEqual(self.output, '\xa0\xc2\xa0')

    def testChangeOutputEncodingBufferFlushing(self):
        buf = self.getRequestBuffer('latin1')
        buf.push_buffer()
        buf.write(unicode('\xa0', 'latin1'))
        self.failUnlessEqual(self.output, '')

        cbuf = util.StringIO()
        buf.push_capture_buffer(cbuf)
        buf.push_buffer()
        buf.write('foo')
        
        buf.set_encoding('utf-8')
        self.failUnlessEqual(buf.encoding, 'utf-8')

        # Test that nothing is flushed.
        self.failUnlessEqual(cbuf.getvalue(), '')
        self.failUnlessEqual(self.output, '')

        buf.pop_buffer()
        buf.pop_capture_buffer()
        
        buf.write(unicode(' \xa0', 'latin1'))
        buf.pop_buffer()
        self.failUnlessEqual(self.output, '\xc2\xa0 \xc2\xa0')

    def testReplaceErrors(self):
        buf = self.getRequestBuffer(errors='replace')
        buf.push_filter(test_filter)
        state = buf.get_state()
        buf.push_buffer()
        buf.write(REPLACEMENT_CHARACTER)
        self.failUnlessEqual(self.output, '')
        buf.pop_to_state(state)
        self.failUnlessEqual(self.output, 'f(?)')
        buf.write(u'a')
        self.failUnlessEqual(self.output, 'f(?)f(a)')
        
    def testFlushWithErrors(self):
        buf = self.getRequestBuffer()
        buf.push_buffer()
        buf.write(unicode('\xa4', 'latin1'))
        self.failUnlessRaises(UnicodeEncodeError, buf.pop_buffer)
        self.failUnlessEqual(self.output, '')
    
    
################################################################

class TestIsAsciiSuperset(unittest.TestCase):
    def test(self):
        self.failUnless(_is_ascii_superset('latin1'))
        self.failUnless(_is_ascii_superset('ascii'))
        self.failUnless(_is_ascii_superset('utf8'))
        self.failIf(_is_ascii_superset('utf7'))
        

        
if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)



# subrequest.py
"""Test subrequests
"""
import StringIO, unittest
from myghty import interp, request
import testbase

class SubrequestTests(testbase.ComponentTester):
    srcFiles = { 'subreq.myt': 'Howdy'
                 }

    def testCapture(self):
        self.runComponent('''
            <%flags> trim = "both" </%flags>
            <%python>
                import StringIO
                buf = StringIO.StringIO()
                subreq = m.create_subrequest("/subreq.myt", out_buffer=buf,
                                             request_args={})
                subreq.execute()
            </%python>
            Subrequest said: "<% buf.getvalue() %>"
            ''')
        self.failUnlessEqual(self.output, 'Subrequest said: "Howdy"')

    def testNoCapture(self):
        self.runComponent('''
            <%flags> trim = "both" </%flags>
            Subrequest said: "<% m.subexec("subreq.myt") or '' %>"
            ''')
        self.failUnlessEqual(self.output, 'Subrequest said: "Howdy"')

    def testSubreq(self):
        self.runComponent("/subreq.myt")
        self.failUnlessEqual(self.output, 'Howdy')

    def testNoFlushOnAbort(self):
        self.runComponent('''
            <%flags> trim = "both" </%flags>
            % m.subexec("subreq.myt")
            % m.abort()
            ''')
        self.failUnlessEqual(self.output, '')

    def testAutoFlush(self):
        self.runComponent('''#
            <%flags>
                autoflush = True
            </%flags>
            % m.subexec("subreq.myt")
            % m.abort()
            ''')
        self.failUnlessEqual(self.output, 'Howdy')
            
    def testDefaultOutputEncoding(self):
        encoding, subreq_encoding = self.runComponent('''
            <%flags> trim = "both" </%flags>
            <%python>
                import StringIO
                buf = StringIO.StringIO()
                subreq = m.create_subrequest("/subreq.myt", out_buffer=buf,
                                             request_args={})
                return ( (m.output_encoding, m.encoding_errors),
                         (subreq.output_encoding, subreq.encoding_errors) )
            </%python>
            ''',
            config={'output_encoding': 'iso-8859-15',
                    'encoding_errors': 'ignore'})
        self.failUnlessEqual(subreq_encoding,
                             (request.DEFAULT_OUTPUT_ENCODING,
                              request.DEFAULT_ENCODING_ERRORS))
        self.failIfEqual(encoding,
                         (request.DEFAULT_OUTPUT_ENCODING,
                          request.DEFAULT_ENCODING_ERRORS))

if __name__ == '__main__':
    unittest.main()



# test/cache_self.py
"""Tests for m.cache_self
"""
import unittest, os, shutil, stat, sys, time, warnings

import testbase
from test_component_reloading import ComponentReloadTests, purge_module

    
class CacheSelfTests(ComponentReloadTests):
    """Abstract base class of tests for component reloading."""

    def setUp(self):
        # Blow away cache
        shutil.rmtree(os.path.join(self.cache, 'container_dbm'),
                      ignore_errors=True)
        #time.sleep(1)
        ComponentReloadTests.setUp(self)

    def reinitializeInterpreter(self):
        params = self.resolverArgs()
        self.module_runner = self.module_runner_class(data_dir=self.cache,
                                                      **params)
        
    def testComponent(self, text="howdy"):
        """Test that we can resolve and execute our component.
        """
        self.failUnlessEqual(self.executeComponent().strip(), text)

    def testCache(self):
        """Test that cache remains intact across interpreter incarnations.
        """
        self.testComponent()
        time.sleep(1)
        self.changeComponentSource(text="changed")

        # Create a new myghty interpreter
        self.reinitializeInterpreter()

        # We should still see the cached value, not the changed value
        self.testComponent()

    def testStaleCache(self):
        """Test that cache gets invalidated if the component source changes.
        """
        #os.system("ls -lR " + self.cache)
        self.testComponent()
        time.sleep(1)
        self.createComponentSource(text="changed")

        # Create a new myghty interpreter
        self.reinitializeInterpreter()
        self.testComponent("changed")

        

################################################################
#
# (The actual unittest.TestCase sub-classes begin here.)
#
################################################################


class FileComponentTests(CacheSelfTests, testbase.MyghtyTest):
    
    def srcFile(self):
        return os.path.join(self.htdocs, "comp.myt")

    def srcText(self, text="howdy"):
        return """
<%%init>
if m.cache_self():
    return
</%%init>
%s
""" % text

    def resolverArgs(self):
        return {'component_root': self.htdocs}

    def componentPath(self):
        return "/comp.myt"
    
class ModuleComponentTests(CacheSelfTests, testbase.MyghtyTest):
    def setUp(self):
        purge_module('module')
        CacheSelfTests.setUp(self)

    def reinitializeInterpreter(self):
        purge_module('module')
        CacheSelfTests.reinitializeInterpreter(self)
        
    def srcFile(self):
        return os.path.join(self.lib, "module.py")

    def srcText(self, text="howdy"):
        return """
def comp(m):
    if not m.cache_self():
        m.write('%s')""" % text

    def resolverArgs(self):
        try:
            del sys.modules['module']
        except KeyError:
            pass
        return {'module_components': [{'/mycomp': 'module:comp'}]}
    
    def componentPath(self):
        return "/mycomp"


if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)



# $Id: cache.py 2062 2006-05-03 05:40:30Z dairiki $
# cache.py - cache API and implementation for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
# 


from myghty.container import *
from myghty.util import *
import myghty.buffer
import re

class Cache:
    """a front-end to the containment API implementing a data cache.
    
    This is a per-request object and is not synchronized against other threads
    or processes.  (the containment API it talks to, is)."""
    

        
    def __init__(self, component, type = None, container_class = None , debug_file = None, **params):

        self.params = CacheArgs()
        self.params.set_params(**params)

        self.type = type    
        if component is not None:
            self.namespace = component.id
        else:
            self.namespace = None
        self.dict = {}

        # attach a ContainerContext to the component, which will hold
        # onto the namespacemanager(s) corresponding to that component.
        # the scope of the context is the same as that of the component,
        # so if a component is unused and gets garbage collected, its 
        # namespacemanagers will get collected as well.
        if component is not None:
            if hasattr(component, "_container_context"):
                self.context = component._container_context
            else:
                self.context = ContainerContext(log_file = debug_file)
                component._container_context = self.context
        
        
        if type is not None:
            self.container_class = container_registry(type, 'Container')
            
        elif container_class is None:
        
            if params.setdefault('data_dir', None) is not None:
                # DBMContainer is definitely faster than FileContainer
                # for caching
                self.container_class = DBMContainer
            else:
                self.container_class = MemoryContainer
                
        else:
            self.container_class = container_class



    
    def _get_container(self, key, **params):
        if not self.dict.has_key(key):
            self.dict[key] = self._create_container(self.namespace, key, **params)

        return self.dict[key]


    def _create_container(self, namespace, key, type = None, container_class = None, **params):
        if container_class is None:
            if type is not None:
                container_class = container_registry(type, 'Container')
            else:
                container_class = self.container_class

        cparams = self.params.get_params(**params)
        
        return container_class(context = self.context, namespace = namespace, key = key, **cparams)
    
        
    def set_container(self, key, **params):
        self.dict[key] = self._create_container(self.namespace, key, **params)
        return self.dict[key]

    def get_container(self, key, **params):
        return self._get_container(key, **params)
        
    def get_value(self, key, **params):
        return self._get_container(key, **params).get_value()

    def set_value(self, key, value, **params):
        self._get_container(key, **params).set_value(value)

    def remove_value(self, key):
        if self.dict.has_key(key):
            self.dict[key].clear_value()
            del self.dict[key]
            
    def clear(self):
        nm = self.get_container(None).get_namespace_manager()
        nm.remove()
        self.dict = {}
        
    # public dict interface
    def __getitem__(self, key):
        return self.get_value(key)
    
    def __contains__(self, key): 
        container = self._get_container(key)
        return container.has_current_value()

    def has_key(self, key): 
        container = self._get_container(key)
        return container.has_current_value()

       
    def __delitem__(self, key):
        self.remove_value(key)
 
    def __setitem__(self, key, value):
        self.set_value(key, value)


class CacheArgs(PrefixArgs):
    
    def __init__(self, **params):
        PrefixArgs.__init__(self, 'cache_')
        self.set_prefix_params(**params)
        
    def clone(self, **params):
        p = self.get_params(**params)
        arg = CacheArgs()
        arg.params = p
        return arg

    def get_cache(self, component, **params):
        return Cache(component, **self.get_params(**params))
        
        



class SimpleController(object):
    """a Myghty controller that delivers a welcome page and a login system."""

    def do_component_init(self, *args, **kwargs):
        pass

    def __call__(self, m, **kwargs):
        """the default callable on this SimpleController.  useful with 
        module-path resolution to produce a result when no further path tokens exist."""
        if m.request_path != '/':
            m.abort(404)
        self.welcome_page(m, **kwargs)        
        
    def welcome_page(self, m, **kwargs):
        m.subexec("index.myt")

    def login(self, m, s, username=None, password=None, cmd=None, **kwargs):
        """displays a login page and processes login information.  note 
        the presence of the 's' argument which represents the session.  the 's'
        variable is configured in the webconfig.py file via 'use_session=True'."""
        if username is not None:
            user = _validate_user(username, password)
            if user is not None:
                s['user'] = user
                s.save()
                self.welcome_page(m, **kwargs)
            else:
                m.subexec("login.myt", username=username, failed=True)
        elif cmd == 'logout':
            del s['user']
            s.save()
            m.subexec("logout.myt")
        else:
            m.subexec("login.myt")
        
# establish an instance of SimpleController as 'index'
index = SimpleController()


# private objects and methods, which are preceded by an underscore
# to mark as private when using the ResolvePathModule resolver

class _User(object):
    """represents a logged-in user."""
    def __init__(self, username, fullname):
        self.username = username
        self.fullname = fullname            


def _validate_user(username, password):
    """given a username and password, returns a valid User object
    or None if no user could be found.  """

    # some stub code.  
    if username == 'testuser' and password == 'password':
        return _User('testuser', 'Test User')
    else:
        return None



__all__ = ['StateMachine', 'StateTransition', 'AbortTransition', 'Transition']

class StateMachine:
    def do_transition(self, state, transition):
        trans = self.__class__.transitions[state + "_" + transition]
        trans.function(self)
        return trans.tostate

class StateTransition:
    def __init__(self, fromstate, tostate, transition, function):
        self.fromstate = fromstate
        self.tostate = tostate
        self.transition = transition
        self.function = function
        self.key = fromstate + "_" + transition

class AbortTransition(Exception):
    pass

def Transition(fromstate, tostate, transition, function):
    trans = StateTransition(fromstate, tostate, transition, function)
    return (trans.key, trans)
    
        